Package: esetVis
Type: Package
Title: Visualizations of expressionSet Bioconductor object
Version: 1.39.0
Date: 2023-12-15
Author: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Maintainer: Laure Cougnaud <laure.cougnaud@openanalytics.eu>
Description: Utility functions for visualization of expressionSet (or
        SummarizedExperiment) Bioconductor object, including spectral
        map, tsne and linear discriminant analysis. Static plot via the
        ggplot2 package or interactive via the ggvis or rbokeh packages
        are available.
Imports: mpm, hexbin, Rtsne, MLP, grid, Biobase, MASS, stats, utils,
        grDevices, methods
Suggests: ggplot2, ggvis, plotly, ggrepel, knitr, rmarkdown, ALL,
        hgu95av2.db, AnnotationDbi, pander, SummarizedExperiment, GO.db
biocViews: Visualization, DataRepresentation, DimensionReduction,
        PrincipalComponent, Pathways
VignetteBuilder: knitr
License: GPL-3
NeedsCompilation: no
RoxygenNote: 7.2.3
git_url: https://git.bioconductor.org/packages/esetVis
git_branch: devel
git_last_commit: 268e44c
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
Packaged: 2026-04-28 22:35:16 UTC; biocbuild

# Generated by roxygen2: do not edit by hand

export(esetLda)
export(esetPlot)
export(esetPlotInteractive)
export(esetPlotWrapper)
export(esetSpectralMap)
export(esetTsne)
export(getGeneSetsForPlot)
export(ggplotEsetPlot)
export(ggvisEsetPlot)
export(plotEset)
export(plotlyEsetPlot)
exportClasses(esetPlot)
exportClasses(esetPlotInteractive)
exportClasses(ggplotEsetPlot)
exportClasses(ggvisEsetPlot)
exportClasses(plotlyEsetPlot)
import(Biobase)
import(mpm)
importFrom(MASS,lda)
importFrom(MLP,getGeneSets)
importFrom(Rtsne,Rtsne)
importFrom(grDevices,colorRamp)
importFrom(grDevices,colorRampPalette)
importFrom(grid,unit)
importFrom(hexbin,hexbin)
importFrom(methods,callNextMethod)
importFrom(methods,new)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,setNames)
importFrom(utils,capture.output)
importFrom(utils,getFromNamespace)
importFrom(utils,packageVersion)

## ----options, echo = FALSE----------------------------------------------------------------------------------------------------------------------------------------------
	
	library(knitr)
	opts_chunk$set(echo = TRUE, results = 'asis', warning = FALSE, 
		error = FALSE, message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		#out.width = '0.7\\textwidth', 
		fig.align = 'center')#, out.height = 0.5\\textwidth', fig.path = 'graphs/') 
	options(width = 170)#, stringsAsFactors = FALSE
	options(warn = 1)#instead of warn = 0 by default -> to have place where warnings occur in the call to Sweave function
	

## ----loadLibraries, results = 'hide', echo = FALSE----------------------------------------------------------------------------------------------------------------------

	library(esetVis)
#	library(xtable)
	library(pander)


## ----ALL-loadAndFormatAllDataset----------------------------------------------------------------------------------------------------------------------------------------

	library(ALL)
	data(ALL)
	
	# to get gene annotation from probe IDs (from the paper HGU95aV2 gene chip was used)
	library("hgu95av2.db")
	library("AnnotationDbi")
	probeIDs <- featureNames(ALL)
	geneInfo <- AnnotationDbi::select(hgu95av2.db, probeIDs, 
		c("ENTREZID", "SYMBOL", "GENENAME"), "PROBEID")
	# 482 on the 12625 probe IDs don't have ENTREZ ID/SYMBOL/GENENAME

	# remove genes with duplicated annotation: 1214
	geneInfoWthtDuplicates <- geneInfo[!duplicated(geneInfo$PROBEID), ]

	# remove genes without annotation: 482
	genesWthtAnnotation <- rowSums(is.na(geneInfoWthtDuplicates)) > 0
	geneInfoWthtDuplicatesAndWithAnnotation <- geneInfoWthtDuplicates[!genesWthtAnnotation, ]
	
	probeIDsWithAnnotation <- featureNames(ALL)[featureNames(ALL) %in% 
		geneInfoWthtDuplicatesAndWithAnnotation$PROBEID]
	ALL <- ALL[probeIDsWithAnnotation, ]
	
	fData <- geneInfoWthtDuplicatesAndWithAnnotation[
		match(probeIDsWithAnnotation, geneInfoWthtDuplicatesAndWithAnnotation$PROBEID), ]
	rownames(fData) <- probeIDsWithAnnotation
	fData(ALL) <- fData

	# grouping variable: B = B-cell, T = T-cell
	groupingVariable <- pData(ALL)$BT
	
	# create custom palette
	colorPalette <- c("dodgerblue", 
		colorRampPalette(c("white","dodgerblue2", "darkblue"))(5)[-1], 
		"red", colorRampPalette(c("white", "red3", "darkred"))(5)[-1])
	names(colorPalette) <- levels(groupingVariable)
	color <- groupingVariable; levels(color) <- colorPalette
	
	# reformat type of the remission
	remissionType <- ifelse(is.na(ALL$remission), "unknown", as.character(ALL$remission))
	ALL$remissionType <- factor(remissionType,
		levels = c("unknown", "CR", "REF"), 
		labels = c("unknown", "achieved", "refractory"))


## ----ALL-loadAndFormatAllDataset-extraNotes, echo = FALSE---------------------------------------------------------------------------------------------------------------

	fvarMetadata(ALL)$labelDescription <- paste(rownames(fvarMetadata(ALL)),
		"obtained from the Bioconductor hgu95av2.db package")	


## ----rmAnnotObjectsFromMemory, echo = FALSE-----------------------------------------------------------------------------------------------------------------------------

	rm(list = c("fData", ls(pattern = "^(geneInfo|probeIDs)")));tmp <- gc(verbose = FALSE)
	#for testing: sort(sapply(ls(), function(x) object.size(get(x))))


## ----ALL-someDataCharacteristics, echo = FALSE--------------------------------------------------------------------------------------------------------------------------

	varLabelsUsed <- c("cod", "sex", "age", "BT", "remissionType")
	pander(head(pData(ALL)[, varLabelsUsed]),
		caption = "subset of the **phenoData** of the ALL dataset for the first genes")
	
	pander(head(fData(ALL)), 
        caption = "**featureData** of the ALL dataset for the first genes")


## ----ALL-esetSimple-----------------------------------------------------------------------------------------------------------------------------------------------------

	print(esetSpectralMap(eset = ALL))


## ----ALL-esetSampleAnnotation1------------------------------------------------------------------------------------------------------------------------------------------

	print(esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Sample annotation (1)",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex", 
		sizeVar = "age", sizeRange = c(0.1, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamples = 0, topGenes = 0, cloudGenes = TRUE))


## ----ALL-esetSampleAnnotation2, fig.height = 8--------------------------------------------------------------------------------------------------------------------------

	gg <- esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Sample annotation (2)",
		colorVar = "age",
		shapeVar = "BT", shape = 15+1:nlevels(ALL$BT),
		sizeVar = "remissionType", size = c(2, 4, 6),
		alphaVar = "age", alphaRange = c(0.2, 0.8),
		topSamples = 0, topGenes = 0, cloudGenes = TRUE)
	# change color of gradient
	library(ggplot2)
	gg <- gg + scale_color_gradientn(colours = 
		colorRampPalette(c("darkgreen", "gold"))(2)
	)
	print(gg)


## ----ALL-customGeneRepresentation---------------------------------------------------------------------------------------------------------------------------------------

	print(esetSpectralMap(eset = ALL,
		psids = 1:1000,
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Custom cloud genes",
		topSamples = 0, topGenes = 0, 
		cloudGenes = TRUE, cloudGenesColor = "red", cloudGenesNBins = 50,
		cloudGenesIncludeLegend = TRUE, cloudGenesTitleLegend = "number of features"))


## ----ALL-genesSamplesAnnotation-----------------------------------------------------------------------------------------------------------------------------------------

	print(esetSpectralMap(eset = ALL, 
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Spectral map \n Label outlying samples and genes"),
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topGenes = 10, topGenesVar = "SYMBOL",
		topGenesCex = 2, topGenesColor = "darkgrey",
		topSamples = 15, topSamplesVar = "cod", topSamplesColor = "chocolate4",
		topSamplesCex = 3
	))


## ----ALL-extractGeneSets------------------------------------------------------------------------------------------------------------------------------------------------

	geneSets <- getGeneSetsForPlot(
		entrezIdentifiers = fData(ALL)$ENTREZID, 
		species = "Human", 
		geneSetSource = 'GOBP',
		useDescription = TRUE)


## ----ALL-geneSetAnnotation----------------------------------------------------------------------------------------------------------------------------------------------

	print(esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Gene set annotation",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topGenes = 0,
		topGeneSets = 4, geneSets = geneSets, geneSetsVar = "ENTREZID", geneSetsMaxNChar = 30,
		topGeneSetsCex = 2, topGeneSetsColor = "purple"))


## ----rmGeneSetsFromMemory, echo = FALSE---------------------------------------------------------------------------------------------------------------------------------

	rm(geneSets);tmp <- gc(verbose = FALSE)


## ----ALL-dimensionsBiPlot-----------------------------------------------------------------------------------------------------------------------------------------------

	print(esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Dimensions of the PCA",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(0.5, 4),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		dim = c(3, 4)))


## ----ALL-interactivePlot-plotly-----------------------------------------------------------------------------------------------------------------------------------------
esetSpectralMap(
	eset = ALL,
	title = paste("Acute lymphoblastic leukemia dataset - spectral map"),
	colorVar = "BT", color = unname(colorPalette),
	shapeVar = "sex",
	alphaVar = "remissionType",
	typePlot = "interactive", packageInteractivity = "plotly",
	figInteractiveSize = c(700, 600),
	topGenes = 10, topGenesVar = "SYMBOL",
	topSamples = 10,
	# use all sample variables for hover
	interactiveTooltipExtraVars = varLabels(ALL)
)

## ----ALL-interactivePlot-ggvis, cache = FALSE---------------------------------------------------------------------------------------------------------------------------
library(ggvis)

# embed a static version of the plot
esetSpectralMap(
	eset = ALL,
	title = paste("Acute lymphoblastic leukemia dataset - spectral map"),
	colorVar = "BT", color = unname(colorPalette),
	shapeVar = "sex",
	alphaVar = "remissionType",
	typePlot = "interactive", packageInteractivity = "ggvis",
	figInteractiveSize = c(700, 600),
	sizeVar = "age", sizeRange = c(2, 6),
	# use all phenoData variables for hover
	interactiveTooltipExtraVars = varLabels(ALL)
)

## ----ALL-tsne-----------------------------------------------------------------------------------------------------------------------------------------------------------

	print(esetTsne(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Tsne",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod"
	))


## ----ALL-tsne-preProcessing---------------------------------------------------------------------------------------------------------------------------------------------

	print(esetTsne(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Tsne",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod",
		fctTransformDataForInputTsne = 
			function(mat)	as.dist((1 - cor(mat))/2)
	))


## ----ALL-Lda, fig.keep = 'first'----------------------------------------------------------------------------------------------------------------------------------------

	# extract random features, because analysis is quite time consuming
	retainedFeatures <- sample(featureNames(ALL), size = floor(nrow(ALL)/5))
	
	# run the analysis
	outputEsetLda <- esetLda(eset = ALL[retainedFeatures, ], ldaVar = "BT",
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Linear discriminant analysis on BT variable \n",
			"(Subset of the feature data)"),
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod", topGenesVar = "SYMBOL",
		returnAnalysis = TRUE
	)

	# extract and print the ggplot object
	print(outputEsetLda$plot)

## ----ALL-Lda-topElements------------------------------------------------------------------------------------------------------------------------------------------------

	# extract top elements labelled in the plot
	pander(t(data.frame(topGenes = outputEsetLda$topElements[["topGenes"]])))
	pander(t(data.frame(topSamples = outputEsetLda$topElements[["topSamples"]])))


## ----ALL-Lda-changeSomeParameters---------------------------------------------------------------------------------------------------------------------------------------

	# to change some plot parameters
	esetPlotWrapper(
		dataPlotSamples = outputEsetLda$analysis$dataPlotSamples,
		dataPlotGenes = outputEsetLda$analysis$dataPlotGenes,
		esetUsed = outputEsetLda$analysis$esetUsed,
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Linear discriminant analysis on BT variable (2) \n",
			"(Subset of the feature data)"),
		colorVar = "BT", color = colorPalette,
		shapeVar = "remissionType", 
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod", topGenesVar = "SYMBOL"
	)


## ----rmOutputEsetLdaFromMemory, echo = FALSE----------------------------------------------------------------------------------------------------------------------------

	rm(outputEsetLda);tmp <- gc(verbose = FALSE)


## ----ALL-Lda-BcellOnly--------------------------------------------------------------------------------------------------------------------------------------------------

	# keep only 'B-cell' samples
	ALLBCell <- ALL[, grep("^B", ALL$BT)]
	ALLBCell$BT <- factor(ALLBCell$BT)
	colorPaletteBCell <- colorPalette[1:nlevels(ALLBCell$BT )]
	
	print(esetLda(eset = ALLBCell[retainedFeatures, ], ldaVar = "BT",
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Linear discriminant analysis on BT variable \n B-cell only",
			"(Subset of the feature data)"
		),
		colorVar = "BT", color = colorPaletteBCell,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod", topGenesVar = "SYMBOL",
	))


## ----rmALLBCellFromMemory, echo = FALSE---------------------------------------------------------------------------------------------------------------------------------

	rm(ALLBCell);tmp <- gc(verbose = FALSE)

#' An S4 class to represent \code{esetPlot} object
#' expressionSet with visualization data from
#' dimension-reduction methods
#'
#' @slot dataPlotSamples data.frame with columns 'X', 'Y' with coordinates 
#' for the samples and with rownames which 
#' should correspond and be in the same order as the sampleNames of
#' \code{esetUsed}
#' @slot dataPlotGenes data.frame with two columns 'X' and 'Y' with 
#' coordinates for the genes
#' @slot eset expressionSet (or SummarizedExperiment) object with data
#' @slot colorVar name of variable (in varLabels of the \code{eset}) used 
#' for coloring, empty by default
#' @slot color character or factor with specified color(s) for the points, 
#' replicated if needed. This is used only if \code{colorVar} is empty.
#' By default: 'black' if \code{colorVar} is not specified and default 
#' \code{ggplot} palette otherwise
#' @slot shapeVar name of variable (in varLabels of the \code{eset}) 
#' used for the shape, empty by default
#' @slot shape character or factor with specified shape(s) (pch) for the points, 
#' replicated if needed. This is used only if \code{shapeVar} is empty.
#' By default: '15' (filled square) if \code{shapeVar} is not specified 
#' and default \code{ggplot} shape(s) otherwise
#' @slot sizeVar name of variable (in varLabels of the \code{eset}) 
#' used for the size, empty by default
#' @slot size size character or factor with specified size(s) (cex) for the points, 
#' replicated if needed.
#' This is used only if \code{sizeVar} is empty.
#' By default: '2.5' if \code{sizeVar} is not specified and default 
#' \code{ggplot} size(s) otherwise
#' @slot sizeRange, size (cex) range used in the plot, possible only 
#' if the \code{sizeVar} is 'numeric' or 'integer'
#' @slot alphaVar name of variable (in varLabels of the \code{eset}) 
#' used for the transparency, empty by default.
#' @slot alpha alpha character or factor with specified transparency(s) for the points,
#' replicated if needed. This is used only if \code{shapeVar} is empty. 
#' By default: '1' if \code{alphaVar} is not specified and default 
#' \code{ggplot} alpha otherwise.
#' @slot alphaRange transparency (alpha) range used in the plot, 
#' possible only if the \code{alphaVar} is 'numeric' or 'integer'
#' @slot symmetryAxes set symmetry for axes, either:
#' \itemize{
#'  \item{'combine' (by default): }{both axes are symmetric and with the same limits}
#'  \item{'separate': }{each axis is symmetric and has its own limits}
#'  \item{'none': }{axes by default (plot limits)}
#' }
#' @slot cloudGenes logical, if TRUE (by default), include the 
#' cloud of genes in the spectral map
#' @slot cloudGenesColor if \code{cloudGenes} is TRUE, 
#' color for the cloud of genes, black by default
#' @slot cloudGenesNBins number of bins to used for the clouds of genes,
#' by default the square root of the number of genes
#' @slot cloudGenesIncludeLegend logical, if TRUE (FALSE by default) 
#' include the legend for the cloud of genes (in the top position if multiple legends)
#' @slot cloudGenesTitleLegend string with title for the legend for the cloud of genes
#' 'nGenes' by default
#' @slot packageTextLabel package used to label the outlying genes/samples/gene sets,
#' either \code{ggrepel} (by default, only used if package \code{ggrepel} is available),
#' or \code{ggplot2}
#' @slot topGenes numeric indicating which percentile (if <1) or number (if >=1) of genes
#' most distant to the origin of the plot to annotate, by default: 10 genes are selected
#' If no genes should be annotated, set this parameter to 0
#' Currently only available for static plot.
#' @slot topGenesCex cex for gene annotation (used when \code{topGenes} > 0)
#' @slot topGenesVar variable of the featureData used to label the genes, 
#' by default: empty, the featureNames are used for labelling (used when \code{topGenes} > 0)
#' @slot topGenesJust text justification for the genes 
#' (used when \code{topGenes} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
#' by default: c(0.5, 0.5) so centered
#' @slot topGenesColor text color for the genes 
#' (used when \code{topGenes} > 0), black by default
#' @slot topSamples numeric indicating which percentile (if <1) or number (if >=1) 
#' of samples most distant to the origin of the plot to annotate, 
#' by default: 10 samples are selected
#' If no samples should be annotated, set this parameter to 0.
#' Currently available for static plot.
#' @slot topSamplesCex cex for sample annotation (used when \code{topSamples} > 0)
#' @slot topSamplesVar variable of the phenoData used to label the samples, 
#' by default: empty, the sampleNames are used for labelling 
#' (used when \code{topSample}s > 0)
#' @slot topSamplesJust text justification for the samples 
#' (used when \code{topSamples} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
#' by default: c(0.5, 0.5) so centered
#' @slot topSamplesColor text color for the samples 
#' (used when \code{topSamples} > 0), black by default
#' @slot geneSets list of gene sets/pathways, each containing 
#' identifiers of genes contained in the set.
#' E.g. pathways from Gene Ontology databases output from the 
#' \code{\link{getGeneSetsForPlot}} function or any custom list of pathways.
#' The genes identifiers should correspond to the variable 
#' \code{geneSetsVar} contained in the phenoData, if not specified
#' the featureNames are used.
#' If several gene sets have the same name, they will be 
#' combine to extract the top gene sets.
#' @slot geneSetsVar variable of the featureData used to 
#' match the genes contained in geneSets,
#' most probably ENTREZID, if not specified the featureNames
#'  of the eSet are used
#' Only used when \code{topGeneSets} > 0 and the parameter 
#' geneSets is specified.
#' @slot geneSetsMaxNChar maximum number of characters for pathway names, 
#' by default keep entire names
#' Only used when \code{topGeneSets} > 0 and the parameter \code{geneSets} is specified.
#' @slot topGeneSets numeric indicating which percentile (if <=1) or number (if >1) of gene sets
#' most distant to the origin of the plot to annotate, by default: 10 gene sets are selected
#' If no gene sets should be annotated, set this parameter to 0.
#' Currently available for static plot.
#' Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.
#' @slot topGeneSetsCex cex for gene sets annotation
#' Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.
#' @slot topGeneSetsJust text justification for the gene sets 
#' by default: c(0.5, 0.5) so centered
#' Only used when \code{topGeneSets} > 0, the parameter \code{geneSets} 
#' is specified and if \code{packageTextLabel} is \code{ggplot2}.
#' @slot topGeneSetsColor color for the gene sets 
#' (used when \code{topGeneSets} > 0 and \code{geneSets} is specified), black by default
#' Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.
#' @slot includeLegend logical if TRUE (by default) include a legend, 
#' otherwise not
#' @slot includeLineOrigin if TRUE (by default) include vertical line at 
#' x = 0 and horizontal line at y = 0
#' @return S4 object of class \code{esetPlot}
#' @import Biobase
#' @name esetPlot-class
#' @rdname esetPlot-class
#' @importFrom methods new
#' @export
esetPlot <- setClass("esetPlot", 
	slots = c(
		eset = "ExpressionSet",
		dataPlotSamples = "data.frame",
		dataPlotGenes = "data.frame",
		cloudGenes = "logical", 
		cloudGenesColor = "character", 
		cloudGenesNBins = "numeric", 		
		cloudGenesIncludeLegend = "logical",
		cloudGenesTitleLegend = "character",
		colorVar = "character", 
		color = "character", 
		shapeVar = "character", 
		shape = "numeric", 
		sizeVar = "character", 
		size = "numeric",
		alphaVar = "character",
		alpha = "numeric",
		sizeRange = "numeric", 
		alphaRange = "numeric",
		includeLineOrigin = "logical",
		symmetryAxes = "character",
		topGenes = "numeric", 
		topGenesVar = "character", 
		topGenesCex = "numeric", 
		topGenesJust = "numeric", 
		topGenesColor = "character", 
		topSamples = "numeric", 
		topSamplesCex = "numeric", 
		topSamplesVar = "character", 
		topSamplesJust = "numeric", 
		topSamplesColor = "character",
		geneSets = "list", 
		geneSetsVar = "character", 
		geneSetsMaxNChar = "numeric", 
		topGeneSets = "numeric",
		topGeneSetsCex = "numeric", 
		topGeneSetsJust = "numeric", 
		topGeneSetsColor = "character",
		includeLegend = "logical",
		packageTextLabel = "character"
	),
	prototype = list(
		cloudGenes = TRUE, 
		cloudGenesColor = "black", 
		cloudGenesIncludeLegend = FALSE,
		cloudGenesTitleLegend = "character",
		color = "black", 
		shape = 15, 
		alpha = 1,
		size = 2.5,
		includeLineOrigin = TRUE,
		symmetryAxes = "combine",
		topGenes = 10, 
		topGenesCex = 2.5, 
		topGenesJust = c(0.5, 0.5), 
		topGenesColor = "black",
		topSamples = 10, 
		topSamplesCex = 2.5, 
		topSamplesJust = c(0.5, 0.5), 
		topSamplesColor = "black",
		topGeneSets = 10,
		topGeneSetsCex = 2.5, 
		topGeneSetsJust = c(0.5, 0.5), 
		topGeneSetsColor = "black",
		packageTextLabel = "ggrepel", 
		includeLegend = TRUE
	)
)

#' constructor for \link{esetPlot}
#' 
#' Constructor of the \link{esetPlot} class
#' @name esetPlot
#' @aliases initialize,esetPlot-method
#' @rdname esetPlot-class
#' @param .Object \link{esetPlot} object
#' @param ... additional class arguments
#' @importFrom methods callNextMethod
#' @keywords internal
setMethod("initialize", "esetPlot", function(.Object, ...) {
			
	.Object <- callNextMethod(.Object, ...)
	if(length(.Object@cloudGenesNBins) == 0 && nrow(.Object@dataPlotGenes) > 0){
		.Object@cloudGenesNBins <- sqrt(nrow(.Object@dataPlotGenes))
	}
	return(.Object)
			
})



#' @title S4 Class Union with character/expression/call
#' @description This is used for the definition of the title/axes labels for the ggplot2 version
#' @keywords internal
setClassUnion(name = "characterORexpressionOrCall", members = c("character", "expression", "call"))

#' a S4 class to represent \code{ggplot} plots
#' @slot returnTopElements logical, if TRUE (FALSE by default) 
#' return the outlying elements labelled in the plot (if any)
#' @slot title string or expression with plot title, '' by default
#' @slot xlab string or expression with label for the x axis
#' @slot ylab string or expression with label for the y axis
#' @return S4 object of class \code{ggplotEsetPlot}
#' @author Laure Cougnaud
#' @name ggplotEsetPlot-class
#' @rdname ggplotEsetPlot-class
#' @importFrom methods new
#' @export
ggplotEsetPlot <- setClass("ggplotEsetPlot", 
	slots = c(
		returnTopElements = "logical",
		title = "characterORexpressionOrCall",
		xlab = "characterORexpressionOrCall", 
		ylab = "characterORexpressionOrCall"
	),
	prototype = list(
		returnTopElements = FALSE,
		title = "",	xlab = "", ylab = ""
	),
	contains = "esetPlot"
)

#' a S4 class to represent interactive plots
#' @slot includeTooltip logical, if TRUE, add hoover functionality showing
#' sample annotation (variables used in the plot) in the plot
#' @slot tooltipVars name of extra phenotypic variable(s) 
#' to add in plotlyEsetPlot to label the samples
#' @slot sizePlot vector containing the size of the interactive plot, 
#' as [width, height], by default: c(600, 400).
#' @slot title string plot title, '' by default
#' @slot xlab string label for the x axis
#' @slot ylab string label for the y axis
#' @return S4 object of class \code{esetPlotInteractive}
#' @author Laure Cougnaud
#' @name esetPlotInteractive-class
#' @rdname esetPlotInteractive-class
#' @export
#' @importFrom methods new
#' @keywords internal
esetPlotInteractive <- setClass("esetPlotInteractive", 
	slots = c(
		includeTooltip = "logical",
		tooltipVars = "character",
		sizePlot = "numeric",
		title = "character", xlab = "character", ylab = "character"
	),
	prototype = list(
		includeTooltip = TRUE,
		sizePlot = c(600, 400),
		title = "",	xlab = "", ylab = ""
	),
	contains = "esetPlot"
)

#' a S4 class to represent \code{plotly} plots
#' @slot returnTopElements logical, if TRUE (FALSE by default)
#' return the outlying elements labelled in the plot (if any)
#' @slot size specified size(s) (cex) for the points, replicated if 
#' needed, used only if \code{sizeVar} is empty, a factor or character
#' by default: '20' if \code{sizeVar} is not specified
#' @return S4 object of class \code{plotlyEsetPlot}
#' @author Laure Cougnaud
#' @name plotlyEsetPlot-class
#' @rdname plotlyEsetPlot-class
#' @export
#' @importFrom methods new
#' @keywords internal
plotlyEsetPlot <- setClass("plotlyEsetPlot",
	slots = list(size = "numeric", returnTopElements = "logical"),
	prototype = list(size = 20, returnTopElements = FALSE,
		topGenesCex = 10, topSamplesCex = 10, topGeneSetsCex = 10),
	contains = "esetPlotInteractive"
)

#' a S4 class for \code{ggvis} plot
#' @slot adjustLegend logical, if TRUE (by default) adjust the legends in \code{ggvis} to avoid
#' overlapping legends when multiple legends
#' @slot alphaRange transparency (alpha) range used in the plot, 
#' c(0.1, 1) by default.
#' @return S4 object of class \code{ggvisEsetPlot}
#' @author Laure Cougnaud
#' @name ggvisEsetPlot-class
#' @rdname ggvisEsetPlot-class
#' @export
#' @importFrom methods new
#' @keywords internal
ggvisEsetPlot <- setClass("ggvisEsetPlot", 
	slots = c("adjustLegend" = "logical"),
	prototype = list(
		adjustLegend = TRUE,
		alphaRange = c(0.1, 1)
	),
	contains = "esetPlotInteractive"
)

#' visualize and \link{esetPlot} with the 'ggplot2' package
#' @param object object of class \link{esetPlot}
#' @return \code{ggplot} object
#' @importFrom grid unit
#' @importFrom utils packageVersion
#' @author Laure Cougnaud
#' @keywords internal
ggPlotEset <- function(object){
		
	if(!requireNamespace("ggplot2", quietly = TRUE))
		stop(paste("The package 'ggplot2' need to be loaded to create static plots."))
	
	
	plotGeneSets <- if(length(object@geneSets) > 0 & object@topGeneSets > 0){
		if(length(object@geneSetsVar) == 0){
			warning(paste("No gene sets are plotted because the variable",
				"describing the mapping of genes IDs between the 'geneSets' and the 'eset'",
				"objects is not specified in the 'geneSetsVar' parameter."
			))
			FALSE
		}else TRUE
	}else FALSE

	# bind data samples with annotation
	dataPlotSamplesWithAnnotation <- getDataPlotSamplesWithAnnotation(object)
	
	# base plot
	g <- ggplot2::ggplot()
	
	## gene plot first
	if(object@cloudGenes & nrow(object@dataPlotGenes) > 0){
				
		# implementation for fill
		baseFillColor <- do.call("rgb", c(as.list(
			colorRamp(c("white", object@cloudGenesColor))(0.2)), list(maxColorValue = 255))
		)
		
		# because parameter/function names changed in ggplot2
		if (utils::packageVersion("ggplot2") < "2.0.0"){
			fillAes <- '..count..'
			statBinHexFct <- ggplot2::stat_binhex
		}else{
			# ggplot2 2.0.0: stat_binhex() has been renamed to stat_bin_hex()
			statBinHexFct <- ggplot2::stat_bin_hex
			# ggplot2 2.1.0: output of stat_bin_hex() is now value instead of count.
			fillAes <- if (utils::packageVersion("ggplot2") == "2.1.0")	'..value..'	else	'..count..'
		}
		
		g <- g + statBinHexFct(
			ggplot2::aes_string(x = 'X', y = 'Y', fill = fillAes),
				data = object@dataPlotGenes, bins = object@cloudGenesNBins) +
			ggplot2::scale_fill_gradientn(colours = c(baseFillColor, object@cloudGenesColor))#c(0.5, 0.8)
		
		#if(includeLegend)	nLegendsSamples <- length(c(colorVar, shapeVar, sizeVar, alphaVar))
		# set legend if specified, and in the first position
		fillGuideArgs <- if(!object@cloudGenesIncludeLegend){
			ifelse(packageVersion("ggplot2") >= "3.3.4", "none", FALSE)
		}else{
			ggplot2::guide_legend(
				order = 1, 
				# change legend title, by default 'count'
				title = object@cloudGenesTitleLegend
			)
		}
		g <- g + ggplot2::guides(fill = fillGuideArgs)
			
		
	}
	
	## sample plot
	
	# fix for ggplot when variable names contained space
	formatVariableSpace <- function(var)
		if(grepl(" ", var))	paste0("`", var, "`")	else var
	
	aesArgSamplePlot <- c(list(x = 'X', y = 'Y'),
		if(length(object@colorVar) > 0)	list(color = formatVariableSpace(object@colorVar)),
		if(length(object@shapeVar) > 0)	list(shape = formatVariableSpace(object@shapeVar)),
		if(length(object@sizeVar) > 0)	list(size = formatVariableSpace(object@sizeVar)),
		if(length(object@alphaVar) > 0)	list(alpha = formatVariableSpace(object@alphaVar))
	)
	
	geomPointArgs <- c(
		list(data = dataPlotSamplesWithAnnotation,
			mapping = do.call(ggplot2::aes_string, aesArgSamplePlot)), 
		if(setFixElement(object@colorVar, object@color))	list(color = object@color),
		if(setFixElement(object@shapeVar, object@shape))	list(shape = object@shape),
		if(setFixElement(object@sizeVar, object@size))	list(size = object@size),
		if(setFixElement(object@alphaVar, object@alpha))	list(alpha = object@alpha)
	)
	
	g <- g + do.call(ggplot2::geom_point, geomPointArgs)
	
	#horizontal/vertical lines
	if(object@includeLineOrigin){
		g <- g + ggplot2::geom_vline(xintercept = 0, linetype = 'dashed')			  # X axis
		g <- g + ggplot2::geom_hline(yintercept = 0, linetype = 'dashed')			  # Y axis
	}
	
	#manual specifications: custom scales
	#only if variable, values are specified and if the variable is not numeric or integer (doesn't work with ggplot2)
	setManualScaleStatic <- function(typeVar, nameVar, valVar){
		values <- formatManualScale(x = dataPlotSamplesWithAnnotation, valVar, nameVar)
		do.call(getFromNamespace(paste("scale", typeVar, "manual", sep = "_"), ns = "ggplot2"),
			list(values = values) )#, name = nameVar, 
	}
	if (setManualScale(dataPlotSamplesWithAnnotation, object@colorVar, object@color))	
		g <- g + setManualScaleStatic(typeVar = "color", nameVar = object@colorVar,
			valVar = object@color)
	
	if (setManualScale(dataPlotSamplesWithAnnotation, object@shapeVar, object@shape))	
		g <- g + setManualScaleStatic("shape", object@shapeVar, object@shape)
	
	if (setManualScale(dataPlotSamplesWithAnnotation, object@sizeVar, object@size))	
		g <- g + setManualScaleStatic("size", object@sizeVar, object@size)
	
	if (setManualScale(dataPlotSamplesWithAnnotation, object@alphaVar, object@alpha))	
		g <- g + setManualScaleStatic("alpha", object@alphaVar, object@alpha)
	
	## axes 
	
	# plot axes labels
	if(object@xlab != "")	g <- g + ggplot2::xlab(paste0("\n", object@xlab))
	if(object@ylab != "")	g <- g + ggplot2::ylab(paste(object@ylab, "\n"))
	
	# increase margin between ticks and axes labels
	getElementText <- function(...)
		if (utils::packageVersion("ggplot2") < "2.0.0")
			ggplot2::element_text(...)	else
			ggplot2::element_text(..., margin = unit(0.5, "lines"))
	
	argsTheme <- c(
		list(
			# labels y axes centered on ticks
			axis.text.y = getElementText(vjust = 5),
			# title x axis further away from the axes labels
			axis.title.x = ggplot2::element_text(vjust = -1)
		),
		# code for previous ggplot2 version
		if (utils::packageVersion("ggplot2") < "2.0.0"){
			list(axis.ticks.margin = unit(0.5, "lines"))
			# code for more recent version
			}else{
				list(axis.text.x = ggplot2::element_text(
					margin = unit(0.5, "lines")))
			}
	)
	g <- g + do.call(ggplot2::theme, argsTheme)
	
	# symmetry
	if(object@symmetryAxes != "none"){
		
		# define axes limits
		axesLimits <- getAxesLimits(object)
		# set axes limits
		g <- g + 
			ggplot2::scale_x_continuous(limits = axesLimits[, "x"]) +
			ggplot2::scale_y_continuous(limits = axesLimits[, "y"])
		
	}
	
	# custom size range, works only if size variable is numeric, or integer
	if(length(object@sizeVar) > 0 &&
		class(dataPlotSamplesWithAnnotation[, object@sizeVar]) %in% c("numeric", "integer") & 
		length(object@sizeRange) > 0)	
		g <- g + ggplot2::scale_size(range = object@sizeRange)
	
	# custom transparency range, works only if alpha variable is numeric, or integer
	if(length(object@alphaVar) > 0 &&
		class(dataPlotSamplesWithAnnotation[, object@alphaVar]) %in% c("numeric", "integer")
			& length(object@alphaRange) > 0)	
		g <- g + ggplot2::scale_alpha(range = object@alphaRange)
	
	# add title
	if((is.character(object@title) && object@title != "") | (is.expression(object@title) & length(object@title) > 0))	
		g <- g + ggplot2::ggtitle(object@title) +
			# in version ggplot2 == 2.2.0 title is left adjusted by default
			ggplot2::theme(plot.title = ggplot2::element_text(hjust = 0.5))
	
	
	# theme
	g <- g + ggplot2::theme_bw()
	
	## annotation top genes/samples, at the end to avoid overlapping with plot
	
	topElements <- NULL
	
	# top genes
	if(object@topGenes > 0 & nrow(object@dataPlotGenes) > 0){
		outputTopGenes <- plotTopElements(
			top = object@topGenes, type = "gene", 
			var = object@topGenesVar, cex = object@topGenesCex, 
			just = object@topGenesJust,
			color = object@topGenesColor,
			dataPlotGenes = object@dataPlotGenes, 
			esetUsed =  object@eset,
			returnTopElements = object@returnTopElements,
			packageTextLabel = object@packageTextLabel
		)
		if(object@returnTopElements){
			topElements <- c(topElements, list(topGenes = outputTopGenes$topElements))
			if(!is.null(outputTopGenes$geomText)){
				g <- g + outputTopGenes$geomText
			}else message("No labels are available for the top genes to annotate so they are not represented in the plot.")
		}else{
			if(!is.null(outputTopGenes))	g <- g + outputTopGenes	else
				message("No labels are available for the top genes to annotate so they are not represented in the plot.")
		}
	}
	
	# top samples
	if(object@topSamples > 0){
		outputTopSamples <- plotTopElements(top = object@topSamples, type = "sample", 
			var = object@topSamplesVar, cex = object@topSamplesCex, just = object@topSamplesJust,
			color = object@topSamplesColor,
			dataPlotSamples = dataPlotSamplesWithAnnotation, esetUsed = object@eset,
			returnTopElements = object@returnTopElements,
			packageTextLabel = object@packageTextLabel)
		if(object@returnTopElements){
			topElements <- c(topElements, list(topSamples = outputTopSamples$topElements))
			g <- g + outputTopSamples$geomText
		}else	g <- g + outputTopSamples
	}
	
	#top gene sets
	if(plotGeneSets){
		outputTopGeneSets <- plotTopElements(
			top = object@topGeneSets, type = "geneSets", 
			cex = object@topGeneSetsCex, just = object@topGeneSetsJust,
			color = object@topGeneSetsColor,
			geneSets = object@geneSets, esetUsed = object@eset, 
			dataPlotGenes = object@dataPlotGenes,
			geneSetsVar = object@geneSetsVar,
			geneSetsMaxNChar = object@geneSetsMaxNChar,
			returnTopElements = object@returnTopElements,
			packageTextLabel = object@packageTextLabel)
		if(object@returnTopElements){
			topElements <- c(topElements, list(topGeneSets = outputTopGeneSets$topElements))
			g <- g + outputTopGeneSets$geomText
		}else	g <- g + outputTopGeneSets
	}
	
	if(!object@includeLegend)	g <- g + ggplot2::theme(legend.position = "none")
	
	#if(!is.null(nColLegend))	guide_legend(ncol = nColLegend)
	
	res <- if(object@returnTopElements & !is.null(topElements)){
		list(topElements = topElements, plot = g)
	}else g
	
	return(res)
	
}

#' plot an \link{plotEset} object
#' @param object object of class \link{esetPlot}
#' @param returnEsetPlot logical, 
#' if TRUE return also the \link{esetPlot} object,
#' such as can be re-use for future call to \link{plotEset}
#' @return the plot object if \code{returnEsetPlot} is FALSE,
#' otherwise a list with 'plot': the plot object
#'  and 'esetPlot': the \link{esetPlot} object
#' @rdname plotEset-methods
#' @docType methods
#' @author Laure Cougnaud
#' @export
setGeneric(name = "plotEset",
	def = function(object, returnEsetPlot = FALSE) {
		standardGeneric("plotEset")
	}
)

#' @aliases plotEset,ggplotEsetPlot-method
#' @rdname plotEset-methods
#' @keywords internal
setMethod("plotEset", "ggplotEsetPlot", function(object, returnEsetPlot){		
	plot <- ggPlotEset(object)
	res <- formatOutput(plot, object, "ggplot", returnEsetPlot)
	return(res)
})

#' @aliases plotEset,ggvisEsetPlot-method
#' @rdname plotEset-methods
#' @keywords internal
setMethod("plotEset", "ggvisEsetPlot", function(object, returnEsetPlot){		
	plot <- ggvisPlotEset(object)
	res <- formatOutput(plot, object, "ggvis", returnEsetPlot)
	return(res)
})

#' @aliases plotEset,plotlyEsetPlot-method
#' @rdname plotEset-methods
#' @keywords internal
setMethod("plotEset", "plotlyEsetPlot", function(object, returnEsetPlot){
	plot <- plotlyPlotEset(object)
	res <- formatOutput(plot, object, "plotly", returnEsetPlot)
	return(res)
})

#' generic for get axes limits
#' @param object \link{plotEset} object
#' @return matrix with limits for axes: columns \code{x} and \code{y}
#' @author Laure Cougnaud
#' @docType methods
#' @rdname getAxesLimits-methods
#' @keywords internal
setGeneric(
	name = "getAxesLimits", 
	def = function(object)	standardGeneric("getAxesLimits")
)

#' @aliases getAxesLimits,esetPlot-method
#' @rdname getAxesLimits-methods
#' @keywords internal
setMethod("getAxesLimits",
	signature = "esetPlot",
	definition = function(object){
			
	# get axes limits, depending on the 'symmetryAxes' parameter
	
	# if no genes plotted, only samples, otherwise use genes
	dataXY <- if((object@cloudGenes | object@topGenes > 0) & nrow(object@dataPlotGenes) > 0)	
		rbind(object@dataPlotSamples[, c('X', 'Y')], object@dataPlotGenes[, c('X', 'Y')])	else	object@dataPlotSamples[, c('X', 'Y')]
	
	# get maximum by axis
	maxCoordByAxis <- apply(abs(dataXY), 2, max)
	
	getAxisLimit <- function(x) c(-x, x)
	
	# define axes limits
	axesLimits <- switch(object@symmetryAxes, 
		'separate' = {
			res <- sapply(maxCoordByAxis, getAxisLimit); 
			colnames(res) <- c("x", "y"); 
			res
		},
		'combine' = sapply(c("x", "y"), function(x) 
			getAxisLimit(max(maxCoordByAxis))),
		
		'none' = {
			res <- apply(dataXY, 2, range);
			colnames(res) <- c("x", "y");
			res
		}
	)
	
	return(axesLimits)
	
})

#' get sample data for plot
#' @param object \link{plotEset} object
#' @return data.frame with 'dataPlotSamples' binded
#' with variables displayed in the plot
#' @author Laure Cougnaud
#' @docType methods
#' @rdname getDataPlotSamplesWithAnnotation-methods
#' @keywords internal
setGeneric(
	name = "getDataPlotSamplesWithAnnotation", 
	def = function(object)	
		standardGeneric("getDataPlotSamplesWithAnnotation")
)

#' @aliases getDataPlotSamplesWithAnnotation,esetPlot-method
#' @rdname getDataPlotSamplesWithAnnotation-methods
#' @keywords internal
setMethod("getDataPlotSamplesWithAnnotation",
	signature = "esetPlot",
	definition = function(object){
			
	esetMethods <- getMethodsInputObjectEsetVis(object@eset)		
		
	sampleAnnotation <- esetMethods$pData(object@eset)[
		rownames(object@dataPlotSamples), 
		unique(c(object@colorVar, object@shapeVar, object@sizeVar, object@alphaVar)), 
		drop = FALSE
	]

	dataPlotWithAnnotation <- cbind(object@dataPlotSamples, sampleAnnotation)
	
	colnames(dataPlotWithAnnotation) <- c(
		colnames(object@dataPlotSamples), 
		colnames(sampleAnnotation)
	)
	
	return(dataPlotWithAnnotation)
	
})

#' @aliases getDataPlotSamplesWithAnnotation,ggvisEsetPlot-method
#' @rdname getDataPlotSamplesWithAnnotation-methods
#' @importFrom methods callNextMethod
#' @keywords internal
setMethod("getDataPlotSamplesWithAnnotation",
	signature = "ggvisEsetPlot",
	definition = function(object){
		
	sampleAnnotation <- callNextMethod(object)	
	
	esetMethods <- getMethodsInputObjectEsetVis(object@eset)
	
	if(object@includeTooltip)
		sampleAnnotation <- cbind(sampleAnnotation, 
			keyggvis = esetMethods$sampleNames(object@eset))

	return(sampleAnnotation)
			
})

#' @aliases getDataPlotSamplesWithAnnotation,plotlyEsetPlot-method
#' @rdname getDataPlotSamplesWithAnnotation-methods
#' @importFrom methods callNextMethod
#' @keywords internal
setMethod("getDataPlotSamplesWithAnnotation",
	signature = "plotlyEsetPlot",
	definition = function(object){

	sampleAnnotation <- callNextMethod(object)

	esetMethods <- getMethodsInputObjectEsetVis(object@eset)

	if(object@includeTooltip & length(object@tooltipVars) > 0){
		sampleAnnotation <- cbind(sampleAnnotation,
			esetMethods$pData(object@eset)[
				rownames(sampleAnnotation),
				setdiff(object@tooltipVars, colnames(sampleAnnotation)),
				drop = FALSE
			]
		)
	}

	return(sampleAnnotation)

})

#' @title get gene sets for plot of \linkS4class{eSet} object.
#' @description get and format gene sets to be used as \code{geneSets} for the functions:
#' \code{\link{esetSpectralMap}}, \code{\link{esetLda}}, or \code{\link{esetPlotWrapper}}
#' Use the \code{\link[MLP]{getGeneSets}} function to get the gene sets,
#' combine all databases, and format the gene sets name if required.
#' @param entrezIdentifiers string with Entrez Gene identifiers of the genes of interest
#' @param species species to use, given to the \code{\link[MLP]{getGeneSets}} function
#' @param geneSetSource gene set source, either 'GOBP', 'GOMF', 'GOCC' or 'KEGG'.
#' Multiple choices are available
#' @param useDescription logical, if TRUE (by default) use the description to 
#' label the gene sets, otherwise use the original gene set identifiers
#' Function 'substr' is used.
#' @param trace logical, if TRUE (by default) 
#' a few extra information are printed during the process
#' @seealso the function used internally: \code{\link[MLP]{getGeneSets}}
#' @return list with gene sets, each element is a gene set and 
#' contains the ENTREZ IDs of the genes contained in this set.
#' If \code{useDescription} is:
#' \itemize{
#' 	\item{FALSE: }{pathways are labelled with identifiers (Gene Ontology IDs for GOBP, GOMF and GOCC, KEGG IDs for KEGG)}
#'  \item{TRUE: }{pathways are labelled with gene sets descriptions}
#' }
#' @examples 
#' # example dataset
#' library(ALL)
#' data(ALL)
#' 
#' # get gene annotation from probe IDs
#' library("hgu95av2.db")
#' probeIDs <- featureNames(ALL)
#' geneInfo <- select(hgu95av2.db, probeIDs,"ENTREZID", "PROBEID")
#' 
#' # get pathway annotation for the genes contained in the ALL dataset (can take a few minutes)
#' geneSets <- getGeneSetsForPlot(entrezIdentifiers = geneInfo$ENTREZID, species = "Human", 
#' 	geneSetSource = 'GOBP',
#' 	useDescription = FALSE, trace = TRUE)
#' head(geneSets) # returns a pathway list of genes
#' 
#' # gene sets labelled with gene sets description
#' geneSets <- getGeneSetsForPlot(entrezIdentifiers = geneInfo$ENTREZID, species = "Human", 
#' 	geneSetSource = 'GOBP', useDescription = TRUE, trace = TRUE)
#' head(geneSets) # returns a pathway list of genes
#' 
#' # see also vignette for an example of the use of this function as input for the esetSpectralMap, esetLda or esetPlotWrapper functions
#' @importFrom MLP getGeneSets
#' @author Laure Cougnaud
#' @export
getGeneSetsForPlot <- function(entrezIdentifiers, species = "Human", 
	geneSetSource = c('GOBP', 'GOMF', 'GOCC', 'KEGG'),
	useDescription = TRUE, trace = TRUE){
	
	geneSetSource <- match.arg(geneSetSource, 
		choices = c('GOBP', 'GOMF', 'GOCC', 'KEGG'), several.ok = TRUE)

	# function from the MLP package, 36 s for 4 databases
	timeGetGeneSets <- system.time(geneSetsList <- lapply(geneSetSource, function(db) 
		getGeneSets(species = species, geneSetSource = db,
			entrezIdentifiers = entrezIdentifiers)))["elapsed"]
	names(geneSetsList) <- geneSetSource
	
	if(trace) message(paste("The extraction of the gene sets for the", length(geneSetSource),
		"databases and", length(entrezIdentifiers), "features", "took:", round(timeGetGeneSets,2), "s."))
	
	if(trace){
		nPathwaysPerDb <- sapply(geneSetsList, length)
		message(paste0("The number of available pathways per database is: ",
			paste(names(nPathwaysPerDb), ": ", nPathwaysPerDb, collapse = ", ", sep = "")
		), ".")
	}
	
	geneSets <- unlist(geneSetsList, recursive = FALSE, use.names = FALSE)
	names(geneSets) <- unlist(sapply(geneSetsList, names))
	
	if(useDescription){
	
		# sometimes more description than names?
		geneSetsDescriptionsList <-  sapply(geneSetsList,
			function(x) attributes(x)$descriptions[attributes(x)$names])
		#sapply(geneSetsDescriptions, length)
		names(geneSets) <- unlist(geneSetsDescriptionsList) 
		# not at this level, otherwise, some pathways can have same names
		#substr(unlist(geneSetsDescriptionsList), 1, nMaxChar)
		
		
	}
	
	return(geneSets)
	
}

#' wrapper to extract useful functions, depending if the object is 
#' an ExpressionSet or a SummarizedExperiment.
#' 
#' This returns an error is \code{x} is not of the correct class.
#' The package \code{SummarizedExperiment} should be available if \code{x} is of class \code{SummarizedExperiment}.
#' @param x object
#' @return if the object is an ExpressionSet or a SummarizedExperiment, 
#' returns a list with the functions specific of the class of \code{x},
#' and equivalent of the ExpressionSet functions: 'sampleNames', 'featureNames',
#' 'fData', 'pData', 'exprs'
#' \itemize{
#' 	 \item{sampleNames: }{sample names}
#'   \item{featureNames: }{feature names}
#'   \item{fData: }{feature annotation}
#'   \item{pData: }{sample annotation}
#'   \item{exprs: }{data matrix}
#'   \item{varLabels: }{sample annotation variables}
#'   \item{fvarLabels: }{feature annotation variables}
#' }
#' @author Laure Cougnaud
#' @import Biobase
#' @keywords internal
getMethodsInputObjectEsetVis <- function(x){
	
	if(inherits(x, what = c('ExpressionSet'))){
		
		return(
			list(
				sampleNames = sampleNames,
				featureNames = featureNames,
				fData = fData, 
				pData = pData,
				exprs = exprs,
				varLabels = varLabels,
				fvarLabels = fvarLabels
			)
		)
		
	}else if(inherits(x, what = c('SummarizedExperiment', 'RangedSummarizedExperiment'))){
		
		if(!requireNamespace("SummarizedExperiment", quietly = TRUE))
			stop(paste("The package 'SummarizedExperiment' need to be loaded to use",
				"the functionalities of esetVis in an 'SummarizedExperiment' object."))
		
		return(
			list(
				sampleNames = colnames,
				featureNames = rownames,
				fData = SummarizedExperiment::rowData, 
				pData = SummarizedExperiment::colData,
				exprs = SummarizedExperiment::assay,
				varLabels = function(x) colnames(SummarizedExperiment::colData(x)),
				fvarLabels = function(x) colnames(SummarizedExperiment::rowData(x))
			)
		)
		
	}else stop("Input object is not of the correct class: 'ExpressionSet' or 'SummarizedExperiment'.")
	
}

#' check if the aesthetic is fixed (e.g. color, shape, size 'palette')
#' @param typeVar name of variable for aesthetic
#' @param valVar fixed value of variable of aesthetic
#' @return logical, if TRUE the element is fixed
#' @author Laure Cougnaud
#' @keywords internal
setFixElement <- function(typeVar, valVar)	(length(typeVar) == 0) & (length(valVar) > 0)

#' check if manual aesthetic should be set
#' 
#' This is the case only if \code{typeVar} and \code{valVar} are specified,
#' and if the variable is not numeric or integer (doesn't work with ggplot2)
#' @param x data.frame with \code{typeVar}
#' @param typeVar name of variable for aesthetic
#' @param valVar fixed value of variable of aesthetic
#' @return logical, if TRUE the manual scale should be set
#' @author Laure Cougnaud
#' @keywords internal
setManualScale <- function(x, typeVar, valVar)	
	(length(typeVar)) > 0 & (length(valVar) > 0) & !class(x[, typeVar]) %in% c("numeric", "integer")

#' extend manual scale values if required
#' @param x data.frame with \code{nameVar}
#' @param valVar fixed value of variable of aesthetic
#' @param nameVar name of variable for aesthetic
#' @return vector of manual scales
#' @author Laure Cougnaud
#' @keywords internal
formatManualScale <- function(x, valVar, nameVar){
	values <- rep(valVar, length.out = nlevels(factor(x[, nameVar])))
	names(values) <- NULL #cannot provide named argument for colors
	values
}

#' capitalize the first letter of a word
#' @param x string
#' @return string with first letter capitalized
#' @keywords internal
simpleCap <- function(x) {
	
	paste0c <- function(...) paste(..., sep = "", collapse = " ")
	paste0c(toupper(substring(x, 1, 1)), substring(x, 2))
	
}

#' format output of \link{plotEset} function
#' @param res result of specific \link{plotEset} function
#' @param object \code{esetPlot} object or extended class
#' @param type string type of plot
#' @param returnEsetPlot logical, should the object be returned in the output function?
#' @return result
#' @author Laure Cougnaud
#' @keywords internal
formatOutput <- function(res, object, type, returnEsetPlot){
	if(returnEsetPlot)
		res <- c(
			if(inherits(res, type))	list(plot = res)	else	res,
			list(esetPlot = object)
		)
	return(res)
}	

#' @title plot a biplot of a linear discriminant analysis of an \linkS4class{eSet} object
#' @description \code{esetLda} reduces the dimension of the data contained in the \linkS4class{eSet} via a linear discriminant analysis
#' on the specified grouping variable with the \code{lda} function and plot the subsequent biplot,
#'  possibly with sample annotation and gene annotation contained in the eSet.
#' @param eset expressionSet (or SummarizedExperiment) object with data
#' @param ldaVar name of variable (in varLabels of the \code{eset}) used for grouping for lda
#' @param psids featureNames of genes to include in the plot, all by default
#' @param dim dimensions of the analysis to represent, first two dimensions by default
#' @param returnAnalysis logical, if TRUE (FALSE by default), return also the output of the analysis,
#' and the outlying samples in the topElements element if any, otherwise only the plot object
#' @inheritParams esetPlotWrapper
#' @references Fisher, R. A. (1936). The Use of Multiple Measurements in
#' Taxonomic Problems. Annals of Eugenics, 7 (2), 179--188
#' @seealso the function used internally: \code{\link[MASS]{lda}}
#' @return if \code{returnAnalysis} is TRUE, return a list:
#' \itemize{
#'  \item{analysis: }{output of the spectral map analysis, whose parameters
#' can be given as input to the \code{\link{esetPlotWrapper}} function}
#'    \itemize{
#' 		\item{dataPlotSamples: }{coordinates of the samples}
#' 		\item{dataPlotGenes: }{coordinates of the genes}
#' 		\item{esetUsed: }{expressionSet used in the plot}
#' 	  }
#'   \item{topElements: }{list with top outlying elements if any, possibly genes, samples and gene sets}
#'   \item{plot: }{the plot output}
#' }
#' otherwise return only the plot	
#' @examples
#' # load data
#' library(ALL)
#' data(ALL)
#' 
#' # specify several variables in ldaVar (this might take a few minutes to run...)
#' 
#' # sample subsetting: currently cannot deal with missing values
#' samplesToRemove <- which(apply(pData(ALL)[, c("sex", "BT")], 1, anyNA)) 
#' 
#' # extract random features, because analysis is quite time consuming
#' retainedFeatures <- sample(featureNames(ALL), size = floor(nrow(ALL)/5))
#' 
#' # create the plot
#' esetLda(eset = ALL[retainedFeatures, -samplesToRemove], 
#'   ldaVar = "BT", colorVar = "BT", shapeVar = "sex", sizeVar = "age",
#'   title = "Linear discriminant analysis on the ALL dataset")
#' @import Biobase
#' @importFrom stats cor
#' @importFrom MASS lda
#' @author Laure Cougnaud
#' @export
esetLda <- function(eset, ldaVar,
	psids = 1:nrow(eset),
	dim = c(1, 2),
	colorVar = character(), 
	color = if(length(colorVar) == 0)	"black"	else	character(),
	shapeVar = character(), 
	shape = if(length(shapeVar) == 0)	15	else	numeric(),
	sizeVar = character(), 
	size = if(length(sizeVar) == 0){
		ifelse(typePlot[1] == "interactive" && packageInteractivity[1] == "plotly", 20, 2.5
		)
	}else{numeric()},
	sizeRange = numeric(), 
	alphaVar = character(), 
	alpha = if(length(alphaVar) == 0)	1	else	numeric(), 
	alphaRange = numeric(),
	title = "", 
	symmetryAxes = c("combine", "separate", "none"),
	packageTextLabel = c("ggrepel", "ggplot2"),
	cloudGenes = TRUE, cloudGenesColor = "black", cloudGenesNBins = sqrt(length(psids)), 
	cloudGenesIncludeLegend = FALSE, cloudGenesTitleLegend = "nGenes",
	topGenes = 10,
	topGenesCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topGenesVar = character(),
	topGenesJust = c(0.5, 0.5), topGenesColor = "black",
	topSamples = 10,
	topSamplesCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topSamplesVar = character(),
	topSamplesJust = c(0.5, 0.5), topSamplesColor = "black",
	geneSets = list(), geneSetsVar = character(), geneSetsMaxNChar = numeric(), 
	topGeneSets = 10, 
	topGeneSetsCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topGeneSetsJust = c(0.5, 0.5), topGeneSetsColor = "black",
	includeLegend = TRUE, includeLineOrigin = TRUE,
	typePlot = c("static", "interactive"), packageInteractivity = c("plotly", "ggvis"),
	figInteractiveSize  = c(600, 400), ggvisAdjustLegend = TRUE, 
	interactiveTooltip = TRUE, interactiveTooltipExtraVars = character(),
	returnAnalysis = FALSE, returnEsetPlot = FALSE){

	if(identical(packageInteractivity, "rbokeh")){
		.Deprecated(msg = paste("The 'rbokeh' interactive plot is deprecated",
			"(as the rbokeh package is archived), a 'plotly' interactive plot",
			"is created instead."
		))
		packageInteractivity <- "plotly"
	}
	
	packageTextLabel <- match.arg(packageTextLabel)
	symmetryAxes <- match.arg(symmetryAxes)
	packageInteractivity <- match.arg(packageInteractivity)
	
	if(length(psids) <= 1)
		stop(paste("At least two genes should be used for the visualization.",
			"Please change the 'psids' argument accordingly."))

	# get methods depending on the class of the object
	esetMethods <- getMethodsInputObjectEsetVis(eset)

	## Extract exprsMat with specified psids
	esetUsed <- eset[psids, ]
	
	errorNAGroupingVariable <- paste("The current implementation cannot deal with",
		"missing values in the grouping variable.",
		"Please remove the corresponding samples from the data."
	)
	
	ldaVariable <- if(length(ldaVar) > 1){
		mesCombinedVar <- "The variables used for lda are combined."
		ldaVariableDf <- esetMethods$pData(esetUsed)[, ldaVar] # pData(esetUsed)
		ldaVariable <- apply(ldaVariableDf, 1, paste, collapse = "-")
		idxNA <- rowSums(is.na(ldaVariableDf)) > 0
		if(sum(idxNA) > 0){
			stop(errorNAGroupingVariable)
#			mesCombinedVar <- paste(mesCombinedVar, 
#				sum(idxNA), "samples with NA in at least one of the variable are set to NA in the combined variable.")
#			ldaVariable[idxNA] <- NA
		}
		message(mesCombinedVar);factor(ldaVariable)
	}else esetMethods$pData(esetUsed)[, ldaVar]
	
	if(!is.factor(ldaVariable))
		stop("The grouping variable for the lda ",
			"should be a factor.")
	
	if(nlevels(ldaVariable) <= 2)
		stop("The current function only deal with ",
			"a grouping variable with at least 3 levels.")

	if(anyNA(ldaVariable))	stop(errorNAGroupingVariable)
		
	## Run lda
	inputLda <- t(esetMethods$exprs(esetUsed)) # exprs(esetUsed)
	system.time(ldaOutput <- lda(inputLda, 
		grouping = ldaVariable))#MASS package

	## reformat results of lda function
	propTraces <- round((ldaOutput$svd^2)/sum(ldaOutput$svd^2)*100)

	if(max(dim) > length(propTraces)){
		warning("The dimensions to represent should be less than",
			" the number of levels in the grouping variable minus 1.",
			" The dimensions c(1, 2) will be used.")
		dim <- c(1, 2)
	}
		
	labelsAxes <- paste0("LD", dim, " (", propTraces[dim],"%)")
	
	#scores of the variables (here genes)
	scoreVar <- ldaOutput$scaling[, dim]
	
	#coordinates of the individuals
	means <- colMeans(ldaOutput$means)
	coordInd <- scale(inputLda, center = means, scale = FALSE) %*% scoreVar
	
	#compute correlation between coordinates of samples and input data
	corrVarInit <- cor(inputLda, coordInd, use = "pairwise")
	# scale it to the maximum limits
	corrVar <- t(t(corrVarInit) * apply(coordInd, 2, max))
#	mostInfluentGenes <- names(sort(apply(corrVar , 1, function(x) sum(abs(x)^2)), decreasing = TRUE)[1:nTopGenes])
#	names(mostInfluentGenes) <- fData(eset)[mostInfluentGenes, "SYMBOL"]
#	corrVarWithoutMostInfluentGenes <- corrVar[!rownames(corrVar) %in% mostInfluentGenes, ]

	## Extract data for final plot
	dataPlotSamples <- data.frame(coordInd, rownames(coordInd))
	colnames(dataPlotSamples) <- c("X", "Y", "sampleName")
	
	dataPlotGenes <- data.frame(corrVar, rownames(corrVar))
	colnames(dataPlotGenes) <- c("X", "Y", "geneName")
	
	plot <- esetVis::esetPlotWrapper(
		dataPlotSamples = dataPlotSamples,
		dataPlotGenes = dataPlotGenes, 
		esetUsed = esetUsed, 
		colorVar = colorVar, color = color,
		shapeVar = shapeVar, shape = shape,
		sizeVar = sizeVar, size = size, sizeRange = sizeRange,
		alphaVar = alphaVar, alpha = alpha, alphaRange = alphaRange,
		title = title, symmetryAxes = symmetryAxes,
		cloudGenes = cloudGenes, cloudGenesColor = cloudGenesColor, 
		cloudGenesNBins = cloudGenesNBins, 
		cloudGenesIncludeLegend = cloudGenesIncludeLegend, cloudGenesTitleLegend = cloudGenesTitleLegend,
		topGenes = topGenes, topGenesCex = topGenesCex, 
		topGenesVar = topGenesVar, topGenesJust = topGenesJust, topGenesColor = topGenesColor,
		topSamples = topSamples, topSamplesCex = topSamplesCex, 
		topSamplesVar = topSamplesVar, topSamplesJust = topSamplesJust, topSamplesColor = topSamplesColor,
		geneSets = geneSets, geneSetsVar = geneSetsVar, geneSetsMaxNChar = geneSetsMaxNChar, 
		topGeneSets = topGeneSets, topGeneSetsCex = topGeneSetsCex, topGeneSetsJust = topGeneSetsJust,
		topGeneSetsColor = topGeneSetsColor,
		includeLegend = includeLegend, includeLineOrigin = includeLineOrigin,
		typePlot = typePlot, packageInteractivity = packageInteractivity,
		figInteractiveSize = figInteractiveSize, 
		ggvisAdjustLegend = ggvisAdjustLegend, interactiveTooltip = interactiveTooltip, interactiveTooltipExtraVars = interactiveTooltipExtraVars,
		returnTopElements = returnAnalysis,
		packageTextLabel = packageTextLabel,
		returnEsetPlot = returnEsetPlot
	)

	res <- if(!returnAnalysis)	plot else
		c(
			list(
				analysis = list(
					dataPlotSamples = dataPlotSamples,
					dataPlotGenes = dataPlotGenes, 
					esetUsed = esetUsed
					#axisLabels = axisLabels
				)
			),
			if(!is.null(plot$topElements))	list(topElements = plot$topElements),
			list(plot = if(inherits(plot, "ggplot"))	plot	else	plot$plot)
		)

	return(res)

}

#' wrapper for biplot of features/samples contained in a \linkS4class{eSet} object
#' 
#' Wrapper function used for all plots of the visualizations contained in the package.
#' @param dataPlotSamples data.frame with columns 'X', 'Y' with coordinates 
#' for the samples and with rownames which 
#' should correspond and be in the same order as the sampleNames of
#' \code{esetUsed}
#' @param dataPlotGenes data.frame with two columns 'X' and 'Y' with 
#' coordinates for the genes
#' @param esetUsed expressionSet (or SummarizedExperiment) object with data
#' @param xlab label for the x axis
#' @param ylab label for the y axis
#' @param colorVar name of variable (in varLabels of the \code{eset}) used 
#' for coloring, empty by default
#' @param color character or factor with specified color(s) for the points, 
#' replicated if needed. This is used only if \code{colorVar} is empty.
#' By default: 'black' if \code{colorVar} is not specified and default 
#' \code{ggplot} palette otherwise
#' @param shapeVar name of variable (in varLabels of the \code{eset}) 
#' used for the shape, empty by default
#' @param shape character or factor with specified shape(s) (pch) for the points, 
#' replicated if needed. This is used only if \code{shapeVar} is empty.
#' By default: '15' (filled square) if \code{shapeVar} is not specified 
#' and default \code{ggplot} shape(s) otherwise
#' @param sizeVar name of variable (in varLabels of the \code{eset}) 
#' used for the size, empty by default
#' @param size character or factor with specified size(s) (cex) for the points, 
#' replicated if needed.
#' This is used only if \code{sizeVar} is empty.
#' By default: '2.5' if \code{sizeVar} is not specified (20 for a plotly plot)
#' and default \code{ggplot} size(s) otherwise
#' @param sizeRange, size (cex) range used in the plot, possible only 
#' if the \code{sizeVar} is 'numeric' or 'integer'
#' @param alphaVar name of variable (in varLabels of the \code{eset}) 
#' used for the transparency, empty by default.
#' This parameter is currently only available for static plot and ggvis (only numeric in this case).
#' @param alpha character or factor with specified transparency(s) for the points,
#' replicated if needed. This is used only if \code{shapeVar} is empty. 
#' By default: '1' if \code{alphaVar} is not specified and default 
#' \code{ggplot} alpha otherwise
#' This parameter is currently only available for static and ggvis.
#' @param alphaRange transparency (alpha) range used in the plot, 
#' possible only if the \code{alphaVar} is 'numeric' or 'integer'
#' This parameter is currently only available for static and ggvis plot.
#' @param title plot title, '' by default
#' @param symmetryAxes set symmetry for axes, either:
#' \itemize{
#'  \item{'combine' (by default): }{both axes are symmetric and with the same limits}
#'  \item{'separate': }{each axis is symmetric and has its own limits}
#'  \item{'none': }{axes by default (plot limits)}
#' }
#' @param cloudGenes logical, if TRUE (by default), include the 
#' cloud of genes in the plot
#' @param cloudGenesColor if \code{cloudGenes} is TRUE, 
#' color for the cloud of genes, black by default
#' @param cloudGenesNBins number of bins to used for the clouds of genes,
#' by default the square root of the number of genes
#' @param cloudGenesIncludeLegend logical, if TRUE (FALSE by default) 
#' include the legend for the cloud of genes (in the top position if multiple legends)
#' @param cloudGenesTitleLegend string with title for the legend for the cloud of genes
#' 'nGenes' by default
#' @param packageTextLabel package used to label the outlying genes/samples/gene sets,
#' either \code{ggrepel} (by default, only used if package \code{ggrepel} is available),
#' or \code{ggplot2}
#' @param topGenes numeric indicating which percentile (if <1) or number (if >=1) of genes
#' most distant to the origin of the plot to annotate, by default: 10 genes are selected
#' If no genes should be annotated, set this parameter to 0
#' Currently only available for static plot.
#' @param topGenesCex cex for gene annotation (used when \code{topGenes} > 0)
#' @param topGenesVar variable of the featureData used to label the genes, 
#' by default: empty, the featureNames are used for labelling (used when \code{topGenes} > 0)
#' @param topGenesJust text justification for the genes 
#' (used when \code{topGenes} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
#' by default: c(0.5, 0.5) so centered
#' @param topGenesColor text color for the genes 
#' (used when \code{topGenes} > 0), black by default
#' @param topSamples numeric indicating which percentile (if <1) or number (if >=1) 
#' of samples most distant to the origin of the plot to annotate, 
#' by default: 10 samples are selected
#' If no samples should be annotated, set this parameter to 0.
#' Currently available for static plot.
#' @param topSamplesCex cex for sample annotation (used when \code{topSamples} > 0)
#' @param topSamplesVar variable of the phenoData used to label the samples, 
#' by default: empty, the sampleNames are used for labelling 
#' (used when \code{topSample}s > 0)
#' @param topSamplesJust text justification for the samples 
#' (used when \code{topSamples} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
#' by default: c(0.5, 0.5) so centered
#' @param topSamplesColor text color for the samples 
#' (used when \code{topSamples} > 0), black by default
#' @param geneSets list of gene sets/pathways, each containing 
#' identifiers of genes contained in the set.
#' E.g. pathways from Gene Ontology databases output from the 
#' \code{\link{getGeneSetsForPlot}} function or any custom list of pathways.
#' The genes identifiers should correspond to the variable 
#' \code{geneSetsVar} contained in the phenoData, if not specified
#' the featureNames are used.
#' If several gene sets have the same name, they will be 
#' combine to extract the top gene sets.
#' @param geneSetsVar variable of the featureData used to 
#' match the genes contained in geneSets,
#' most probably ENTREZID, if not specified the featureNames
#'  of the eSet are used
#' Only used when \code{topGeneSets} > 0 and the parameter 
#' geneSets is specified.
#' @param geneSetsMaxNChar maximum number of characters for pathway names, 
#' by default keep entire names
#' Only used when \code{topGeneSets} > 0 and the parameter \code{geneSets} is specified.
#' If \code{returnAnalysis} is set to TRUE and \code{geneSetsMaxNChar} specified, 
#' the top pathways will be returned in the output object, named with the identifiers used in the plot 
#' (so with maximum \code{geneSetsMaxNChar} number of characters)
#' @param topGeneSets numeric indicating which percentile (if <=1) or number (if >1) of gene sets
#' most distant to the origin of the plot to annotate, by default: 10 gene sets are selected
#' If no gene sets should be annotated, set this parameter to 0.
#' Currently available for static plot.
#' Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.
#' @param topGeneSetsCex cex for gene sets annotation
#' Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.
#' @param topGeneSetsJust text justification for the gene sets 
#' by default: c(0.5, 0.5) so centered
#' Only used when \code{topGeneSets} > 0, the parameter \code{geneSets} 
#' is specified and if \code{packageTextLabel} is \code{ggplot2}.
#' @param topGeneSetsColor color for the gene sets 
#' (used when \code{topGeneSets} > 0 and \code{geneSets} is specified), black by default
#' Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.
#' @param includeLegend logical if TRUE (by default) include a legend, 
#' otherwise not
#' @param includeLineOrigin if TRUE (by default) include vertical line at 
#' x = 0 and horizontal line at y = 0
#' @param typePlot type of the plot returned, either 'static' (static) or 
#' interactive' (potentially interactive)
#' @param figInteractiveSize vector containing the size of the interactive plot, 
#' as [width, height]
#' by default: c(600, 400). This is passed to the \code{width} and 
#' \code{height} parameters of:
#' \itemize{
#'  \item{for plotly plots: }{the \code{\link[plotly]{ggplotly}} function}
#'  \item{for ggvis plots: }{the \code{ggvis::set_options} function}
#' }
#' @param ggvisAdjustLegend logical, if TRUE (by default) adjust the legends in \code{ggvis} to avoid
#' overlapping legends when multiple legends
#' @param interactiveTooltip logical, if TRUE, add hoover functionality showing
#' sample annotation (variables used in the plot) in the plot
#' @param interactiveTooltipExtraVars name of extra variable(s) 
#' (in varLabels of the \code{eset}) to add in plotlyEsetPlot to label the samples,
#' empty by default
#' @param packageInteractivity if \code{typePlot} is 'interactive', 
#' package used for interactive plot,
#' either 'plotly' (by default) (by default) or 'ggvis'.
#' @param returnTopElements logical, if TRUE return also the top elements
#' @param returnEsetPlot logical, if TRUE return also the \link{esetPlot} object
#' @examples
#' library(ALL)
#' data(ALL)
#' 
#' ## run one spectral map analysis
#' 
#' # create custom color palette
#' colorPalette <- c("dodgerblue", colorRampPalette(c("white","dodgerblue2", "darkblue"))(5)[-1], 
#' 	"red", colorRampPalette(c("white", "red3", "darkred"))(5)[-1])
#' 
#' # run the analysis
#' # with 'returnAnalysis' set to TRUE to have all objects required for the esetPlotWrapper
#' outputEsetSPM <- esetSpectralMap(eset = ALL, 
#' 	title = "Acute lymphoblastic leukemia dataset \n Spectral map complete",
#' 	colorVar = "BT", color = colorPalette,
#' 	shapeVar = "sex", shape = 15:16,
#' 	sizeVar = "age", sizeRange = c(2, 6),
#' 	symmetryAxes = "separate",
#' 	topGenes = 10, topGenesJust = c(1, 0), topGenesCex = 2, topGenesColor = "darkgrey",
#' 	topSamples = 15, topSamplesVar = "cod", topSamplesColor = "black",
#' 	topSamplesJust = c(1, 0), topSamplesCex = 3, returnAnalysis = TRUE)
#' 
#' # plot the biplot
#' print(outputEsetSPM$plot)
#' 
#' 
#' ## re-call the plot function, to change some visualizations parameters
#' esetPlotWrapper(
#' 	dataPlotSamples = outputEsetSPM$analysis$dataPlotSamples,
#' 	dataPlotGenes = outputEsetSPM$analysis$dataPlotGenes,
#' 	esetUsed = outputEsetSPM$analysis$esetUsed,
#' 	title = paste("Acute lymphoblastic leukemia dataset \n Spectral map"),
#' 	colorVar = "BT", color = colorPalette,
#' 	shapeVar = "relapse", 
#' 	sizeVar = "age", sizeRange = c(2, 6),
#' 	topSamplesVar = "cod", topGenesVar = "SYMBOL"
#' )
#' @return if \code{typePlot} is:
#' \itemize{
#' 	 \item{\code{static}: }
#'    \itemize{
#'     \item{if \code{returnTopElements} is TRUE, and top elements can be displayed, a list with:}
#'      \itemize{
#'       \item{'topElements': }{the top elements labelled in the plot}
#' 	     \item{'plot': }{the \code{ggplot} object}
#'       }
#'     \item{otherwise, the \code{ggplot} object only}
#'    }
#'   \item{\code{interactive}: }{a \code{ggvis} or \code{plotly} object,
#' 		depending on the \code{packageInteractivity} parameter}
#' }
#' @import Biobase
#' @importFrom hexbin hexbin
#' @importFrom grDevices colorRamp colorRampPalette
#' @importFrom utils getFromNamespace
#' @author Laure Cougnaud
#' @export
esetPlotWrapper <- function(
	dataPlotSamples, 
	dataPlotGenes = data.frame(), esetUsed, 
	xlab = "", ylab = "",
	colorVar = character(0), 
	color = if(length(colorVar) == 0)	"black"	else	character(0),
	shapeVar = character(0), 
	shape = if(length(shapeVar) == 0)	15	else	numeric(0),
	sizeVar = character(0), 
	size = if(length(sizeVar) == 0){
		ifelse(typePlot[1] == "interactive" && packageInteractivity[1] == "plotly", 20, 2.5
		)
	}else{numeric()},
	sizeRange = numeric(0), #c(1, 6),
	alphaVar = character(0), 
	alpha = if(length(alphaVar) == 0)	1	else	numeric(0), 
	alphaRange = numeric(0),
	title = "", 
	#assume that data are already log-transformed
	symmetryAxes = c("combine", "separate", "none"),
	cloudGenes = TRUE, cloudGenesColor = "black",
	cloudGenesNBins = if(nrow(dataPlotGenes) > 0)	sqrt(nrow(dataPlotGenes))	else	numeric(),
	cloudGenesIncludeLegend = FALSE, cloudGenesTitleLegend = "nGenes",
	packageTextLabel = c("ggrepel", "ggplot2"),
	topGenes = 10,
	topGenesCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topGenesVar = character(0), topGenesJust = c(0.5, 0.5), topGenesColor = "black",
	topSamples = 10, topSamplesCex = 2.5, topSamplesVar = character(0), topSamplesJust = c(0.5, 0.5), topSamplesColor = "black",
	geneSets = list(), geneSetsVar = character(0), geneSetsMaxNChar = numeric(0), topGeneSets = 10, 
	topGeneSetsCex = 2.5, topGeneSetsJust = c(0.5, 0.5), topGeneSetsColor = "black",
	includeLegend = TRUE, includeLineOrigin = TRUE,
	typePlot = c("static", "interactive"),
	figInteractiveSize  = c(600, 400), ggvisAdjustLegend = TRUE,
	interactiveTooltip = TRUE, interactiveTooltipExtraVars = character(0),
	packageInteractivity = c("plotly", "ggvis"),
	returnTopElements = FALSE, returnEsetPlot = FALSE){

	if(identical(packageInteractivity, "rbokeh")){
		.Deprecated(msg = paste("The 'rbokeh' interactive plot is deprecated",
			"(as the rbokeh package is archived), a 'plotly' interactive plot",
			"is created instead."
		))
		packageInteractivity <- "plotly"
	}

	symmetryAxes <- match.arg(symmetryAxes)
	packageTextLabel <- match.arg(packageTextLabel)
	packageInteractivity <- match.arg(packageInteractivity)
	typePlot <- match.arg(typePlot)
	
	# get methods depending on the class of the object
#	esetMethods <- getMethodsInputObjectEsetVis(esetUsed)
	
	# add needed variables in the input data for the plot
	## Bug #7160 fix - added drop = FALSE to handle cases with only one 'Var'
		
	## functions common between the different type of plots available

	esetPlotArgs <- list(
		eset = esetUsed,
		dataPlotSamples = dataPlotSamples,
		dataPlotGenes = dataPlotGenes,
		cloudGenes = cloudGenes, 
		cloudGenesColor = cloudGenesColor, 
		cloudGenesNBins = cloudGenesNBins, 
		cloudGenesIncludeLegend = cloudGenesIncludeLegend,
		cloudGenesTitleLegend = cloudGenesTitleLegend,
		colorVar = colorVar, 
		color = color, 
		shapeVar = shapeVar, 
		shape = shape, 
		sizeVar = sizeVar , 
		size = size, 
		alphaVar = alphaVar,
		alpha = alpha,
		sizeRange = sizeRange, 
		alphaRange = alphaRange,
		includeLineOrigin = includeLineOrigin,
		title = title,
		xlab = xlab, ylab = ylab, 
		symmetryAxes = symmetryAxes,
		topGenes = topGenes, 
		topGenesVar = topGenesVar, 
		topGenesCex = topGenesCex, 
		topGenesJust = topGenesJust, 
		topGenesColor = topGenesColor, 
		topSamples = topSamples, 
		topSamplesCex = topSamplesCex, 
		topSamplesVar = topSamplesVar, 
		topSamplesJust = topSamplesJust, 
		topSamplesColor = topSamplesColor,
		geneSets = geneSets, 
		geneSetsVar = geneSetsVar, 
		geneSetsMaxNChar = geneSetsMaxNChar, 
		topGeneSets = topGeneSets,
		topGeneSetsCex = topGeneSetsCex, 
		topGeneSetsJust = topGeneSetsJust, 
		topGeneSetsColor = topGeneSetsColor,
		includeLegend = includeLegend,
		packageTextLabel = packageTextLabel
	)
	
	esetPlotArgs <- c(esetPlotArgs,
			
		switch(typePlot,
	
			'static' = {
				list(returnTopElements = returnTopElements)
			},
				
			'interactive' = {
			
				c(
						
					list(
						includeTooltip = interactiveTooltip, 
						tooltipVars = interactiveTooltipExtraVars, 
						sizePlot = figInteractiveSize
					),
				
					if(packageInteractivity == "ggvis")
						list(adjustLegend = ggvisAdjustLegend)
					
				)
				
			}
		
		)
		
	)
	
	esetPlotFct <- switch(typePlot,
		"static" = "ggplotEsetPlot",
		"interactive" = switch(packageInteractivity,
			'plotly' = "plotlyEsetPlot",
			'ggvis' = "ggvisEsetPlot"
		)
	)
	inputPlot <- do.call(esetPlotFct, esetPlotArgs)
	
	res <- plotEset(inputPlot, returnEsetPlot = returnEsetPlot)

	return(res)
	
}

#' plot top elements for a static plot
#'
#' This create \code{geom_text} object with top genes/sample/pathways
#' @param packageTextLabel package used to label the outlying genes/samples/gene sets,
#' either 'ggrepel' (by default, only used if package \code{ggrepel} is available),
#' or 'ggplot2'
#' @param cex cex of text in the plot
#' @param just justification of elements in the plot, only use if \code{packageTextLabel} is 'ggplot2'
#' @param color color for the elements in the plot
#' @param returnTopElements logical if TRUE (FALSE by default) return the outlying elements
#' @inheritParams getTopElements
#' @return 
#' \itemize{
#'  \item{if the \code{elements} are present in the data: if \code{returnTopElements} is: }
#'	 \itemize{
#'    \item{TRUE: }{return a list with two arguments:} 
#'      \itemize{
#'       \item{topElements: }{string with top elements labelled in the plot}
#'       \item{geomText: }{output of \code{geom_text}}
#'      }
#'     \item{FALSE: }{only return the output of \code{geom_text}} 
#' 	 }
#'  \item{if not, return \code{NULL}}
#' }
#' @keywords internal
#' @author Laure Cougnaud
plotTopElements <- function(
	packageTextLabel = c("ggrepel", "ggplot2"),
	cex = 1, just = c(0.5, 0.5), color = "black",
	returnTopElements = FALSE, ...){

	packageTextLabel <- match.arg(packageTextLabel)
	dataPlotText <- getTopElements(...)

	# issue if all labels are NA
	geomText <- if(all(is.na(dataPlotText$labels)))	NULL	else{

		argsGeomTextFct <- list(
			mapping = ggplot2::aes_string(x = 'X', y = 'Y', label = 'labels'),
			data = dataPlotText, color = color, size = cex
		)
		if(packageTextLabel	== "ggrepel" & requireNamespace("ggrepel", quietly = TRUE)){
			geomTextFct <- ggrepel::geom_text_repel
		}else{
			geomTextFct <- ggplot2::geom_text
			argsGeomTextFct <- c(argsGeomTextFct, list(hjust = just[1], vjust = just[2]))
		}
		do.call(geomTextFct, argsGeomTextFct)

	}

	res <- if(returnTopElements){

		topElements <- dataPlotText$labels
		names(topElements) <- rownames(dataPlotText)
		list(topElements = topElements, geomText = geomText)

	}else	geomText

	return(res)

}

#' create \code{geom_text} object with top genes/sample/pathways
#' @param top numeric, number of top elements
#' @param type type of elements to plot, either 'gene', 'sample', or 'geneSets'
#' @param var variable used to annotate the elements, only used for 'gene' and 'sample'
#' @param dataPlotGenes data.frame with two columns 'X' and 'Y' with coordinates for the genes
#' @param dataPlotSamples data.frame with two columns 'X' and 'Y' with coordinates for the samples
#' @param esetUsed expressionSet (or SummarizedExperiment) object with data
#' @param geneSets list of gene sets, e.g. gene pathways, output from the 'getGeneSets' function in MLP
#' the genes IDs must correspond to the sampleNames in the eset.
#' If several gene sets have the same name, they will be combine to extract the top gene sets.
#' @param geneSetsVar variable of the featureData used to match the genes contained in geneSets,
#' most probably ENTREZID, if not specified the featureNames of the eSet are used
#' @param geneSetsMaxNChar maximum number of characters for pathway names, by default keep entire names
#' @param returnTopElements logical if TRUE (FALSE by default) return the outlying elements
#' @return Data.frame with coordinates and labels of the top elements
#' @keywords internal
#' @author Laure Cougnaud
getTopElements <- function(top,
	type = c("gene", "sample", "geneSets"),
	var = character(),
	dataPlotGenes = data.frame(), dataPlotSamples = data.frame(), esetUsed, 
	geneSets = list(), geneSetsVar = character(), geneSetsMaxNChar = numeric()){

	type <- match.arg(type)
	
	# get methods depending on the class of the object
	esetMethods <- getMethodsInputObjectEsetVis(esetUsed)
	
	switch(type,
			
		'geneSets' = if(length(geneSets) == 0|nrow(dataPlotGenes) == 0)
			stop("`geneSets' and some 'dataPlotGenes' should be specified."),
		
		'gene' = if(nrow(dataPlotGenes) == 0)
			stop("'dataPlotGenes' should be specified."),
		
		'sample' = if(nrow(dataPlotSamples) == 0)
			stop("'dataPlotSamples' should be specified.")

	)
	
	coor <- switch(type, 
			
		#if gene sets, take the mean coordinates for each gene set
		#TODO: improve speed
		'geneSets' = getCoordGeneSets(
			dataPlotGenes = dataPlotGenes, geneSets = geneSets, 
			esetUsed = esetUsed, geneSetsVar = geneSetsVar
		),
		
		#if gene or samples, take directly the coordinates
		switch(type, 'sample' = dataPlotSamples, 'gene' = dataPlotGenes)[, c("X", "Y")]

	)
	
	# for gene sets, if no mapped gene is found
	dataPlotText <- if(nrow(coor) > 0){
	
		nElements <- nrow(coor)
		distToOrigin <- sqrt(rowSums(coor ^ 2))
		idxElementsSorted <- order(distToOrigin, decreasing = TRUE)
		
		#if top less than 1, percentage, otherwise absolute number
		idxTop <- 1:(if(top < 1)	(top * nElements)	else	min(top, nElements))
		idxElementsKept <- idxElementsSorted[idxTop]
		coorElementsKept <- coor[idxElementsKept, ]
	
		labels <- if(type != "geneSets"){
			varInAnnot <- ifelse(length(var) == 0, "", var) %in% 
				switch(type, 'gene' = esetMethods$fvarLabels, 'sample' = esetMethods$varLabels)(esetUsed)
			if(!varInAnnot)	rownames(coorElementsKept)	else
				switch(type, 'gene' = esetMethods$fData, 'sample' = esetMethods$pData)(esetUsed)[
					rownames(coorElementsKept), var]
		}else{
			if(length(geneSetsMaxNChar) > 0){
				res <- rownames(coorElementsKept)
				names(res) <- substr(rownames(coorElementsKept), 1, geneSetsMaxNChar)
				res
			}else	rownames(coorElementsKept)
		}
		
		data.frame(coor[idxElementsKept, ], labels, stringsAsFactors = FALSE)
		
	}else{
		warning(paste("No labels for the", type, "are found."))
		NULL
	}

	return(dataPlotText)
	
}

#' extract coordinates gene sets
#' @param dataPlotGenes data.frame with two columns 'X' and 'Y' with coordinates for the genes
#' @param geneSets geneSets list of gene sets, e.g. gene pathways, output from the 'getGeneSets' function in MLP
#' the genes IDs must correspond to the sampleNames in the eset
#' @param geneSetsVar variable of the featureData used to match the genes contained in geneSets,
#' most probably ENTREZID, if NULL the featureNames of the eSet are used
#' @param ... Any parameters passed to the \code{\link{getTopElements}}
#' @param esetUsed expressionSet (or SummarizedExperiment) object with data
#' @return data.frame with two columns 'X' and 'Y' with coordinates for the gene sets
#' @author Laure Cougnaud
#' @import Biobase
#' @keywords internal
#' @author Laure Cougnaud
getCoordGeneSets <- function(dataPlotGenes, geneSets, esetUsed, geneSetsVar = list()){
	
	# get methods depending on the class of the object
	esetMethods <- getMethodsInputObjectEsetVis(esetUsed)
	
	#previously only, but very slow
#system.time(test <- as.data.frame(t(sapply(geneSets, function(x) 
#		colMeans(dataPlotGenes[matchGeneID(x), c("X", "Y")], na.rm = TRUE)))))
	
	if(any(duplicated(names(geneSets))))
		warning("Some gene sets have the same name, they will be combined ",
				"for the extraction of the top gene sets.")
	
	# convert to vector
	system.time(geneSetsVect <- unlist(geneSets, recursive = FALSE, use.names = TRUE)) # 1s
	names(geneSetsVect) <- rep(names(geneSets), times = sapply(geneSets, length))
	
	# match with gene ID
	useVarIDToMatch <- if(length(geneSetsVar) > 0)	
		geneSetsVar %in% esetMethods$fvarLabels(esetUsed)	else	FALSE
	system.time(geneSetsVectInEset <- esetMethods$featureNames(esetUsed)[
		match(geneSetsVect, 
			if(useVarIDToMatch)	esetMethods$fData(esetUsed)[, geneSetsVar]	else
				esetMethods$featureNames(esetUsed)
		)
	])
	names(geneSetsVectInEset) <- names(geneSetsVect)
	
	# filtered the ones not in eset
	geneSetsVectInEsetFiltered <- geneSetsVectInEset[!is.na(geneSetsVectInEset)]
	
	# extract coordinates
	system.time(dataPlotGenesGS <- dataPlotGenes[geneSetsVectInEsetFiltered, c("X", "Y")]) # 150s
	
	# means by coordinates
	getCoordGeneSets <- function(colName)
		tapply(dataPlotGenesGS[, colName], names(geneSetsVectInEsetFiltered), mean)
	
	system.time(coordGeneSets <- cbind(X = getCoordGeneSets("X"), Y = getCoordGeneSets("Y")))
	
	return(coordGeneSets)
}

#' visualize and \link{esetPlot} with the the 'ggvis' package
#' @param object object of class \link{esetPlot}
#' @return \code{ggvis} plot object
#' @author Laure Cougnaud
#' @keywords internal
ggvisPlotEset <- function(object){
	
	esetMethods <- getMethodsInputObjectEsetVis(object@eset)
	
	if(!requireNamespace("ggvis", quietly = TRUE))
		stop(paste("The package 'ggvis' need to be loaded to create",
			"interactive plots with ggvis."))
	
	# bind data samples with annotation
	dataPlotSamplesWithAnnotation <- getDataPlotSamplesWithAnnotation(object)

	includeCloudGenes <- object@cloudGenes & length(object@dataPlotGenes) > 0
	
	# ggvis doesn't allow different variable type for same aes, here transparency of cloud of genes is numeric and sample can be a factor
	alphaVar <- if(includeCloudGenes && length(object@alphaVar) > 0 && !is.numeric(dataPlotSamplesWithAnnotation[, object@alphaVar])){
		warning("A factor variable for transparency cannot be used while cloud of genes is required so 'shapeVar' is ignored.")
		character(0)
	}else	object@alphaVar
	
	## sample plot
	getProp <- function(type, typeVar)
		if(length(typeVar) > 0)	list(ggvis::prop(type, as.name(typeVar)))
	
	ggvisArgsSamplePlot <- c(
		list(data = dataPlotSamplesWithAnnotation),
		getProp("x", "X"),
		getProp("y", "Y"),
		getProp("fill", object@colorVar),
		getProp("shape", object@shapeVar),
		getProp("size", object@sizeVar),
		getProp("opacity", alphaVar)
		#if(interactiveTooltip)	list(props(key := ~"keyggvis"))
	)
	
	## gene plot first
	if(includeCloudGenes){
		
		hexbinGeneData <- hexbin(
			object@dataPlotGenes$X, 
			object@dataPlotGenes$Y, 
			xbins = object@cloudGenesNBins
		)
		hexbinGeneDataDf <- data.frame(
			xGene = hexbinGeneData@xcm, 
			yGene = hexbinGeneData@ycm, 
			geneCol = hexbinGeneData@count
		)
		
		g <- ggvis::ggvis(x = ~xGene, y= ~yGene, data = hexbinGeneDataDf)
		g <- ggvis::layer_points(vis = g, fillOpacity = ~geneCol, fill = object@cloudGenesColor)
		g <- ggvis::hide_legend(vis = g, c("fillOpacity", "fill")) 
		g <- ggvis::scale_numeric(vis = g, property = "fillOpacity", trans = "sqrt")
		
		# add sample plot to the gene plot
		ggvisArgsSamplePlotWithGenePlot <- c(list(vis = g), ggvisArgsSamplePlot)
		g <- do.call(ggvis::layer_points, ggvisArgsSamplePlotWithGenePlot)
		
	}else	g <- ggvis::layer_points(vis = do.call(ggvis::ggvis, ggvisArgsSamplePlot))
	
	#TODO: horizontal/vertical lines
	#			g <- g + geom_vline(xintercept = 0, linetype = 'dashed')			  # X axis
	#			g <- g + geom_hline(yintercept = 0, linetype = 'dashed')			  # Y axis
	
	# add fixed elements
	if(setFixElement(object@colorVar, object@color))
		g <- ggvis::layer_points(g, fill := object@color)
	if(setFixElement(object@shapeVar, object@shape))
		g <- ggvis::layer_points(g, shape := object@shape)
	if(setFixElement(object@sizeVar, object@size))
		g <- ggvis::layer_points(g, size := object@size)
	if(setFixElement(alphaVar, object@alpha))
		g <- ggvis::layer_points(g, opacity := object@alpha)

	setManualScaleGgvis <- function(g, typeVar, nameVar, valVar, typeScale = "logical"){
		values <- if(length(valVar) > 0)	
			formatManualScale(x = dataPlotSamplesWithAnnotation, valVar, nameVar)
		do.call(
			getFromNamespace(paste0("scale_", typeScale), ns = "ggvis"), 
			list(vis = g, property = typeVar, range = values)
		)
	}	
	

	
	# manual specifications: custom scales
	if (setManualScale(dataPlotSamplesWithAnnotation, object@colorVar, object@color))	
		g <- setManualScaleGgvis(g, typeVar = "fill", nameVar = object@colorVar, valVar = object@color)
	if (setManualScale(dataPlotSamplesWithAnnotation, object@shapeVar, object@shape))	
		g <- setManualScaleGgvis(g, typeVar = "shape", nameVar = object@shapeVar, valVar = object@shape)
	if (setManualScale(dataPlotSamplesWithAnnotation, object@sizeVar, object@size))	
		g <- setManualScaleGgvis(g, typeVar = "size", nameVar = object@sizeVar, valVar = object@size)
	
	# need to specify range for transparence
	alphaRange <- if(length(object@alphaRange) == 0){
		c(0.1, 1)
	}else	object@alphaRange
	if (length(alphaVar) > 0)
		g <- ggvis::scale_ordinal(vis = g, property = "opacity", range = alphaRange)
	
	#plot axes labels and title
	g <- ggvis::add_axis(vis = g, "x", title = object@xlab)
	g <- ggvis::add_axis(vis = g, "y", title = object@ylab)
	
	#symmetry
	if(object@symmetryAxes != "none"){
		
		#define axes limits
		axesLimits <- getAxesLimits(object)
		#set axes limits
		g <- ggvis::scale_numeric(vis = g, "x", trans = "linear", domain = axesLimits[, "x"])
		g <- ggvis::scale_numeric(vis = g, "y", trans = "linear", domain = axesLimits[, "y"])
		
	}
	
	#TODO: custom size range, works only if size variable is numeric, or integer
	#			if(class(dataPlotWithAnnotation[, sizeVar]) %in% c("numeric", "integer")
	#				& !is.null(sizeRange))	g <- g + scale_size(range = sizeRange)
	
	#add title
	if (object@title != ""){
		g <- ggvis::add_axis(
			vis = g, "x", orient = "top", ticks = 0, 
			title = object@title,
			properties = ggvis::axis_props(
				axis = list(stroke = "white"),
				labels = list(fontSize = 2)
			)
		)
	}
	
	figInteractiveSize <- if(length(object@sizePlot) != 2){
		warning(paste("The size of the ggvis window should contain two",
			"elements: width and height. The default values c(600, 400) are used."))
		c(600, 400)
		# default options
		# str(default_options())
	}else object@sizePlot
	
	g <- ggvis::set_options(vis = g, 
		width = figInteractiveSize[1], 
		height = figInteractiveSize[2]
	)
	
#	orderLegendLog <- c(!is.null(colorVar), !is.null(shapeVar), !is.null(sizeVar))
#	names(orderLegendLog) <- c("fill", "shape", "size")
	#			if(sum(orderLegendLog) > 0){
	#				legendToSet <- names(orderLegendLog)[orderLegendLog]
	#				argsAddLegend <- c(as.list(legendToSet), list(orient = "right"))
	#				g <- g %>% do.call("add_legend", argsAddLegend)
	#			}
	
	# adjust legend manually because overlap if several present
	# only if the window size is numeric (not auto)
	# this will be fixed in future version of ggvis? 
	
	orderLegendLog <- c(
		length(object@colorVar) > 0, 
		length(object@shapeVar) > 0, 
		length(object@sizeVar) > 0
	)
	names(orderLegendLog) <- c("fill", "shape", "size")
	legendToSet <- names(orderLegendLog)[orderLegendLog]
	
	if(is.numeric(figInteractiveSize) & 
		object@includeLegend & 
		object@adjustLegend & 
		length(legendToSet) > 0){
		
		# for legend side by side in y direction
		setLegendPos <- function(vis, typeVar, y)
			ggvis::add_legend(vis = vis, scales = typeVar, 
				properties = ggvis::legend_props(legend = list(y = y)))
		
		for(i in 1:length(legendToSet))
			g <- setLegendPos(vis = g, typeVar = legendToSet[i],
				y = (i-1) * figInteractiveSize[2]/length(legendToSet))
		
		# for legend side by side in x direction
		#					setLegendPos <- function(vis, typeVar, x)
		#						add_legend(vis = vis, scales = typeVar, 
		#								properties = legend_props(legend = list(x = x)))
		#					
		#					for(i in 1:length(legendToSet))
		#						g <- setLegendPos(g, typeVar = legendToSet[i],
		#							x = (0.1*(i-1) + 1) * figInteractiveSize[1])
		#				
	}
	
	
	# hide legend
	if((!object@includeLegend) & length(legendToSet) > 0)	
		g <- ggvis::hide_legend(vis = g, scales = legendToSet)
	
	if(object@includeTooltip){
		
		tooltipVars <- object@tooltipVars[
			object@tooltipVars %in% esetMethods$varLabels(object@eset)
		]
		
		if(length(tooltipVars) == 0)
			tooltipVars <- NULL
		
		tooltipLabelsFct <- function(x)
			return(
					
				if(!is.null(x) && !any(c("xGene", "yGene") %in% names(x))){
						
					formatCoor <- function(x)
						paste0("(", paste(formatC(x, width = 3), collapse = ", "), ")")
					
					xy <- as.numeric(x[, c("X", "Y")])
					xyST <- formatCoor(xy)
					
					# use bracket rather than subset ta avoid having notes in R CMD check
					dataPoints <- dataPlotSamplesWithAnnotation[
						round(dataPlotSamplesWithAnnotation$X, 3) == round(xy[1], 3) & 
							round(dataPlotSamplesWithAnnotation$Y, 3) == round(xy[2], 3), 
					]
					
					if(nrow(dataPoints) > 0 & length(tooltipVars) > 0){
						addedAnnot <- esetMethods$pData(object@eset)[dataPoints[1, "sampleName"], 
							tooltipVars, drop = FALSE]
						colnames(addedAnnot) <- tooltipVars
						x <- data.frame(x, addedAnnot)
					}
					
					extraCols <- colnames(x)[!colnames(x) %in% c("X", "Y")]
					
					if(length(extraCols) > 0){
						sampleAnnot <- paste0(names(x[, extraCols]), ": ", x[, extraCols], collapse = "<br />")
						paste0(xyST, "<br />", sampleAnnot)
					}else	xyST
				}
			)
		
			g <- ggvis::add_tooltip(vis = g, tooltipLabelsFct, "hover") # can use 'click' too
		
	}
		
	## TODO: annotation top genes/samples, at the end to avoid overlapping with plot
	
	# to keep legend separated
	g <- ggvis::set_options(vis = g, duration = 0)
	
	return(g)
	
}

#' @title plot a spectral map biplot of an \linkS4class{eSet}.
#' @description \code{esetSpectralMap} reduces the dimension of the data contained in the \linkS4class{eSet} with the \code{\link[mpm]{mpm}} function
#'  and plot the subsequent biplot of the specified dimensions, possibly with gene and sample annotation contained in the \linkS4class{eSet}.
#' A spectral map with the default parameters is equivalent to a 
#' principal component analysis on the log-transformed, double centered and 
#' global normalized data (from documentation of the \code{\link[mpm]{mpm}} function).
#' @param eset expressionSet (or SummarizedExperiment) object with data
#' @param psids featureNames of genes to include in the plot, all by default
#' @param dim dimensions of the analysis to represent, first two dimensions by default
#' @param mpm.args list with input parameters for the \code{\link[mpm]{mpm}} function.
#' The default value is:
#' \code{list(closure = 'none', center = 'double', normal = 'global', 'row.weight' = 'mean',
#' col.weight = 'constant', logtrans = FALSE)}.
#' This assumes that the data are already in a log scale.
#' @param plot.mpm.args list with input parameters for the \code{\link[mpm]{plot.mpm}} function.
#' The default value is: list(scale = "uvc").
#' @param returnAnalysis logical, if TRUE (FALSE by default), return also the output of the analysis,
#' and the outlying samples in the topElements element if any, otherwise only the plot object
#' @inheritParams esetPlotWrapper
#' @references Lewi, P.J. (1976). Spectral mapping, a technique for
#' classifying biological activity profiles of chemical compounds.
#' Arzneimittel Forschung (Drug Research), 26, 1295--1300
#' @seealso the function used internally: \code{\link[mpm]{mpm}} and 
#' \code{\link[a4Base]{spectralMap}} for spectral map in base R graphics
#' @examples
#' library(ALL)
#' data(ALL)
#' 
#' ## complete example (most of the parameters are optional)
#' # create custom color palette
#' colorPalette <- c("dodgerblue", colorRampPalette(c("white","dodgerblue2", "darkblue"))(5)[-1], 
#'		"red", colorRampPalette(c("white", "red3", "darkred"))(5)[-1])
#' # plot the spectral map
#' print(esetSpectralMap(eset = ALL, 
#' 	title = "Acute lymphoblastic leukemia dataset \n Spectral map complete",
#'	colorVar = "BT", color = colorPalette,
#'	shapeVar = "sex", shape = 15:16,
#'	sizeVar = "age", sizeRange = c(2, 6),
#'	symmetryAxes = "separate",
#'	topGenes = 10, topGenesJust = c(1, 0), topGenesCex = 2, topGenesColor = "darkgrey",
#'	topSamples = 15, topSamplesVar = "cod", topSamplesColor = "black",
#'	topSamplesJust = c(1, 0), topSamplesCex = 3)
#')
#' 
#' # see vignette for other examples, especially one with gene sets specification
#' 
#' @return if \code{returnAnalysis} is TRUE, return a list:
#' \itemize{
#'  \item{analysis: }{output of the spectral map analysis, can be given as input to the \code{esetPlotWrapper} function}
#'    \itemize{
#' 		\item{dataPlotSamples: }{coordinates of the samples}
#' 		\item{dataPlotGenes: }{coordinates of the genes}
#' 		\item{esetUsed: }{expressionSet used in the plot}
#'      \item{axisLabels: }{axes labels indicating percentage of variance explained by the selected axes}
#'      \item{axesContributionsPercentages: }percentages of variance explained by each axis (not only the ones specified in \code{dim})
#' 	  }
#'   \item{topElements: }{list with top outlying elements if any, possibly genes, samples and gene sets}
#'   \item{plot: }{the plot output}
#' }
#' otherwise return only the plot
#' @author Laure Cougnaud
#' @import Biobase mpm
#' @export
esetSpectralMap <- function(eset, 
	psids = 1:nrow(eset),
	dim = c(1, 2),
	colorVar = character(),
	color = if(length(colorVar) == 0)	"black"	else	character(),
	shapeVar = character(), 
	shape = if(length(shapeVar) == 0)	15	else	numeric(),
	sizeVar = character(),
	size = if(length(sizeVar) == 0){
		ifelse(typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
			20, 2.5
		)
	}else{numeric()},
	sizeRange = numeric(), #c(1, 6),
	alphaVar = character(), 
	alpha = if(length(alphaVar) == 0)	1	else	numeric(),  
	alphaRange = numeric(),
	title = "", 
	# assume that data are already log-transformed
	mpm.args = list(closure = "none", center = "double", 
		normal = "global", row.weight = "mean", col.weight = "constant",
		logtrans = FALSE),
	plot.mpm.args = list(scale = "uvc"),#except (x, dim, do.plot)
	symmetryAxes = c("combine", "separate", "none"),
	packageTextLabel = c("ggrepel", "ggplot2"),
	cloudGenes = TRUE, cloudGenesColor = "black",
	cloudGenesNBins = sqrt(length(psids)),
	cloudGenesIncludeLegend = FALSE, cloudGenesTitleLegend = "nGenes",
	topGenes = 10,
	topGenesCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topGenesVar = character(), topGenesJust = c(0.5, 0.5), topGenesColor = "black",
	topSamples = 10,
	topSamplesCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topSamplesVar = character(),
	topSamplesJust = c(0.5, 0.5), topSamplesColor = "black",
	geneSets = list(), geneSetsVar = character(), geneSetsMaxNChar = numeric(),
	topGeneSets = 10,
	topGeneSetsCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topGeneSetsJust = c(0.5, 0.5),
	topGeneSetsColor = "black",
	includeLegend = TRUE, includeLineOrigin = TRUE,
	typePlot = c("static", "interactive"),
	packageInteractivity = c("plotly", "ggvis"),
	figInteractiveSize  = c(600, 400), ggvisAdjustLegend = TRUE,
	interactiveTooltip = TRUE, interactiveTooltipExtraVars = character(),
	returnAnalysis = FALSE, returnEsetPlot = FALSE){

	if(identical(packageInteractivity, "rbokeh")){
		.Deprecated(msg = paste("The 'rbokeh' interactive plot is deprecated",
			"(as the rbokeh package is archived), a 'plotly' interactive plot",
			"is created instead."
		))
		packageInteractivity <- "plotly"
	}

	symmetryAxes <- match.arg(symmetryAxes)
	packageTextLabel <- match.arg(packageTextLabel)
	packageInteractivity <- match.arg(packageInteractivity)
	
	if(length(psids) <= 1)
		stop(paste("At least two genes should be used for the visualization.",
			"Please change the 'psids' argument accordingly."))

	# get methods depending on the class of the object
	esetMethods <- getMethodsInputObjectEsetVis(eset)
	
	## Extract exprsMat with specified psids
	esetUsed <- eset[psids, ]

	## Create dataframe for plotting
	exprsMat <- esetMethods$exprs(esetUsed) #exprs
	dataMPM <- data.frame(rownames(exprsMat), exprsMat, stringsAsFactors = FALSE)
	colnames(dataMPM) <- c("featureID", colnames(exprsMat))
	dataMPMWthtNA <- dataMPM[rowSums(is.na(dataMPM)) == 0, ]
	
	## Run mpm with default parameters 
	argsMpm <- c(mpm.args, list("data" = dataMPMWthtNA))
	mpmResults <- do.call("mpm", argsMpm)
	
	## Convert spm results into plotting coordinates
	argsPlotMpm <- c(plot.mpm.args, 
		list("x" = mpmResults, "do.plot" = FALSE, "dim" = dim))
		#labels = fData(eset)$SYMBOL
	mpmCoords <- do.call("plot.mpm", argsPlotMpm)

	## Extract data for final plot
	dataPlotSamples <- data.frame(mpmCoords$Columns[, c("X", "Y")],
		sampleName = rownames(mpmCoords$Columns))

	dataPlotGenes <- data.frame(mpmCoords$Rows[, c("X", "Y")],
		geneName = rownames(mpmCoords$Rows))

	## label axes
	#extract percentages of variance in each axis
	pctVar <- round(mpmResults$contrib*100)
	axisLabels <- paste0("PC", dim, ": ", pctVar[dim], "%")

	## Plot
	plot <- esetVis::esetPlotWrapper(
		dataPlotSamples = dataPlotSamples,
		dataPlotGenes = dataPlotGenes, 
		esetUsed = esetUsed, 
		xlab = axisLabels[1], ylab = axisLabels[2],
		colorVar = colorVar, color = color,
		shapeVar = shapeVar, shape = shape,
		sizeVar = sizeVar, size = size, sizeRange = sizeRange,
		alphaVar = alphaVar, alpha = alpha, alphaRange = alphaRange,
		title = title, symmetryAxes = symmetryAxes,
		cloudGenes = cloudGenes, cloudGenesColor = cloudGenesColor, 
		cloudGenesNBins = cloudGenesNBins, 
		cloudGenesIncludeLegend = cloudGenesIncludeLegend, cloudGenesTitleLegend = cloudGenesTitleLegend,
		topGenes = topGenes, topGenesCex = topGenesCex, topGenesVar = topGenesVar, 
		topGenesJust = topGenesJust, topGenesColor = topGenesColor,
		topSamples = topSamples, topSamplesCex = topSamplesCex, 
		topSamplesVar = topSamplesVar, topSamplesJust = topSamplesJust, topSamplesColor = topSamplesColor,
		geneSets = geneSets, geneSetsVar = geneSetsVar, geneSetsMaxNChar = geneSetsMaxNChar, 
		topGeneSets = topGeneSets, topGeneSetsCex = topGeneSetsCex, topGeneSetsJust = topGeneSetsJust,
		topGeneSetsColor = topGeneSetsColor, 
		includeLegend = includeLegend, includeLineOrigin = includeLineOrigin,
		#nColLegend = nColLegend,
		typePlot = typePlot, 
		figInteractiveSize = figInteractiveSize, ggvisAdjustLegend = ggvisAdjustLegend, interactiveTooltip = interactiveTooltip, 
		interactiveTooltipExtraVars = interactiveTooltipExtraVars,
		returnTopElements = returnAnalysis,
		packageInteractivity = packageInteractivity,
		packageTextLabel = packageTextLabel,
		returnEsetPlot = returnEsetPlot)

	res <- if(!returnAnalysis)	plot else
		c(
			list(
				analysis = list(
					dataPlotSamples = dataPlotSamples,
					dataPlotGenes = dataPlotGenes, 
					esetUsed = esetUsed,
					axisLabels = axisLabels,
					axesContributionsPercentages = mpmResults$contrib*100
				)
			),
			if(!is.null(plot$topElements))	list(topElements = plot$topElements),
			list(plot = if(inherits(plot, "ggplot"))	plot	else	plot$plot)
		)

	return(res)
	
}

#' visualize and \link{esetPlot} with the the 'plotly' package
#' @param object object of class \link{esetPlot}
#' @return \code{plotly} plot
#' @author Laure Cougnaud
#' @importFrom stats setNames
#' @keywords internal
plotlyPlotEset <- function(object){

	esetMethods <- getMethodsInputObjectEsetVis(object@eset)

	if(!requireNamespace("plotly", quietly = TRUE))
		stop(paste("The package 'plotly' need to be loaded to create",
			"interactive plots with plotly."))

	## base plot

	figInteractiveSize <- if(length(object@sizePlot) != 2){
		warning(paste("The size of the window should contain two",
			"elements: width and height. The default values c(600, 400) are used."))
			c(600, 400)
	}else object@sizePlot
	argsPlotly <- c(
		list(width = figInteractiveSize[1], height = figInteractiveSize[2]),
		# colors/shapes should be set in plot_ly call, not add_trace
		if(length(object@colorVar) > 0 && length(object@color) > 0)
			list(colors = object@color),
		if(length(object@shapeVar) > 0 && length(object@shape) > 0)
			list(symbols = object@shape),
		if(length(object@sizeVar) > 0 && length(object@size) > 0)
			list(sizes = object@size)
	)
	fig <- do.call(plotly::plot_ly, argsPlotly)

	## add gene cloud
	if(object@cloudGenes & nrow(object@dataPlotGenes) > 0){

		# create data.frame with x/y position of hexbin + counts
		dataBin <- hexbin::hexbin(
			x = object@dataPlotGenes$X, y = object@dataPlotGenes$Y,
			xbins = object@cloudGenesNBins
		)
		dataBinDf <- do.call(cbind.data.frame, hexbin::hcell2xy(dataBin))
		cBin <- dataBin@count
		dataBinDf[, "count"] <- cBin/max(cBin)

		# colors
		baseFillColor <- do.call("rgb", c(as.list(
			colorRamp(c("white", object@cloudGenesColor))(0.2)),
			list(maxColorValue = 255))
		)
		colorscale <- list(list(0, baseFillColor), list(1, object@cloudGenesColor))

		# create hexbin plot
		fig <- plotly::add_markers(
			p = fig, data = dataBinDf, x = ~ x, y = ~ y,
			type = "scatter", showlegend = FALSE,
			hoverinfo = "none",
			marker = list(color = ~ count, colorscale = colorscale)
		)

	}

	# add samples
	dataSamples <- getDataPlotSamplesWithAnnotation(object)
	setAes <- function(var, value, type){
		isVar <- !is.null(var) && length(var) > 0
		x <- list(
			if(isVar){varToFm(var)}else{I(value)}
		)
		names(x) <- type
		x <- x[!sapply(x, is.null)]
		return(x)
	}
	argsSamplePlot <- c(
		list(
			p = fig, x = varToFm("X"), y = varToFm("Y"),
			type = "scatter", mode = "markers", showlegend = TRUE
		),
		setAes(var = object@colorVar, value = object@color, type = "color"),
		setAes(var = object@sizeVar, value = object@size, type = "size"),
		setAes(var = object@shapeVar, value = object@shape, type = "symbol"),
		setAes(var = object@alphaVar, value = object@alpha, type = "opacity")
	)
	if(object@includeTooltip & length(object@tooltipVars)){
		hovertext <- lapply(object@tooltipVars, function(var)
			paste0(var, ": ", dataSamples[, var])
		)
		dataSamples[, "hovertext"] <- do.call(what = paste,
			args = c(hovertext, list(sep = "<br>")))
		argsSamplePlot[["hovertemplate"]] <- varToFm("hovertext")
	}
	argsSamplePlot[["data"]] <- dataSamples
	fig <- do.call(plotly::add_trace, argsSamplePlot)

	topElements <- NULL

	# add top genes
	if(object@topGenes > 0 & nrow(object@dataPlotGenes) > 0){

		# get coordinates of top elements
		dataTopGenes <- getTopElements(
			top = object@topGenes, type = "gene",
			var = object@topGenesVar,
			dataPlotGenes = object@dataPlotGenes,
			esetUsed =  object@eset
		)

		# plot top elements
		fig <- plotly::add_text(
			p = fig, x = ~X, y = ~Y, text = ~labels,
			data = dataTopGenes,
			textfont = list(
				color = object@topGenesColor,
				size = object@topGenesCex
			),
			showlegend = FALSE
#			textposition = object@topGenesJust
		)

		if(object@returnTopElements){
			topGenes <- setNames(dataTopGenes$labels, rownames(dataTopGenes))
			topElements <- c(topElements, list(topGenes = topGenes))
		}

	}

	if(object@topSamples > 0 & nrow(object@dataPlotSamples) > 0){

		# get coordinates of top elements
		dataTopSamples <- getTopElements(
			top = object@topSamples, type = "sample",
			var = object@topSamplesVar,
			dataPlotSamples = object@dataPlotSamples,
			esetUsed =  object@eset
		)

		# plot top elements
		fig <- plotly::add_text(
			p = fig, x = ~X, y = ~Y, text = ~labels,
			data = dataTopSamples,
			textfont = list(
				color = object@topSamplesColor,
				size = object@topSamplesCex
			),
			showlegend = FALSE
#			textposition = object@topSamplesJust
		)

		if(object@returnTopElements){
			topSamples <- setNames(dataTopSamples$labels, rownames(dataTopSamples))
			topElements <- c(topElements, list(topSamples = topGenes))
		}

	}

	# top gene sets
	plotGeneSets <- if(length(object@geneSets) > 0 & object@topGeneSets > 0){
		if(length(object@geneSetsVar) == 0){
			warning(paste("No gene sets are plotted because the variable",
				"describing the mapping of genes IDs between the 'geneSets' and the 'eset'",
				"objects is not specified in the 'geneSetsVar' parameter."
				))
			FALSE
		}else TRUE
	}else FALSE

	if(plotGeneSets){

		# get coordinates of top elements
		dataTopGeneSets <- getTopElements(
			top = object@topGeneSets, type = "geneSets",
			dataPlotGenes = object@dataPlotGenes,
			geneSets = object@geneSets,
			esetUsed =  object@eset,
			geneSetsVar = object@geneSetsVar,
			geneSetsMaxNChar = object@geneSetsMaxNChar
		)

		# plot top elements
		fig <- plotly::add_text(
			p = fig, x = ~X, y = ~Y, text = ~labels,
			data = dataTopGeneSets,
			textfont = list(
				color = object@topGeneSetsColor,
				size = object@topGeneSetsCex
			),
			showlegend = FALSE
#			textposition = object@topSamplesJust
		)

		if(object@returnTopElements){
			topGeneSets <- setNames(dataTopGeneSets$labels, rownames(dataTopGeneSets))
			topElements <- c(topElements, list(topGeneSets = topGeneSets))
		}

	}

	## axes
	argsLayout <- list()

	# title
	if(object@title != "")	argsLayout$title$text <- object@title

	# plot axes labels
	if(object@xlab != "")	argsLayout$xaxis$title$text <- object@xlab
	if(object@ylab != "")	argsLayout$yaxis$title$text <- object@ylab

	# symmetry
	if(object@symmetryAxes != "none"){

		# define axes limits
		axesLimits <- getAxesLimits(object)
		# set axes limits
		argsLayout$xaxis$range <- axesLimits[, "x"]
		argsLayout$yaxis$range <- axesLimits[, "y"]

	}

	if(length(argsLayout) > 0){
		argsLayout[["p"]] <- fig
		fig <- do.call(plotly::layout, argsLayout)
	}

	return(fig)

}

#' Get formula for a specific variable,
#' to be used in aesthetic specification in \code{\link[plotly]{plot_ly}}.
#' @param var Character vector with variable to combine.
#' Otherwise with the '+' operator.
#' @return \code{\link[stats]{as.formula}}
#' @author Laure Cougnaud
#' @importFrom stats as.formula
#' @keywords internal
varToFm <- function(var){
	fm <- as.formula(paste0("~", paste(paste0("`", var, "`"), collapse = "+")))
	return(fm)
}

#' @title plot a t-SNE of an \linkS4class{eSet} object
#' @description \code{esetTsne} reduces the dimension of the data contained in the \linkS4class{eSet} via t-Distributed Stochastic Neighbor Embedding
#'  with the \code{\link[Rtsne]{Rtsne}} function and plot the subsequent biplot, possibly with sample annotation contained in the eSet.
#' @param eset expressionSet (or SummarizedExperiment) object with data
#' @param psids featureNames of genes to include in the plot, all by default
#' @param trace logical, if TRUE (by default), print some messages during tsne is running
#' @param Rtsne.args arguments for the Rtsne function, by default:
#' perplexite parameter = optimal number of neighbours, 
#' theta = speed/accuracy trade-off (increase for less accuracy), set to 0.0 for exact TSNE
#' @param fctTransformDataForInputTsne function which transform the data in the eSet object before
#' calling the \code{\link[Rtsne]{Rtsne}} function. 
#' This should be a function which takes a matrix as input and return a matrix, e.g. the dist function.
#' @param returnAnalysis logical, if TRUE (FALSE by default), return also the output of the analysis,
#' and the outlying samples in the topElements element if any, otherwise only the plot object
#' @inheritParams esetPlotWrapper
#' @references L.J.P. van der Maaten and G.E. Hinton (2008). Visualizing
#' High-Dimensional Data Using t-SNE. Journal of Machine Learning
#' Research, 2579--2605
#' @seealso the function used internally: \code{\link[Rtsne]{Rtsne}} or \url{http://homepage.tudelft.nl/19j49/t-SNE.html}
#'  for further explanations about this technique.
#' @return if \code{returnAnalysis} is TRUE, return a list:
#' \itemize{
#'  \item{analysis: }{output of the spectral map analysis, whose elements can be given
#'  to the \code{\link{esetPlotWrapper}} function}
#'    \itemize{
#' 		\item{dataPlotSamples: }{coordinates of the samples}
#' 		\item{esetUsed: }{expressionSet used in the plot}
#' 	  }
#'   \item{topElements: }{list with top outlying elements if any, possibly genes, samples and gene sets}
#'   \item{plot: }{the plot output}
#' }
#' otherwise return only the plot
#' @examples 
#' library(ALL)
#' data(ALL)
#' 
#' ## complete example (most of the parameters are optional)
#' 
#' # create custom color palette
#' colorPalette <- c("dodgerblue", colorRampPalette(c("white","dodgerblue2", "darkblue"))(5)[-1], 
#'		"red", colorRampPalette(c("white", "red3", "darkred"))(5)[-1])
#' 
#' # create tsne
#' print(esetTsne(eset = ALL, 
#' 	title = "Acute lymphoblastic leukemia dataset \n Tsne complete",
#' 	colorVar = "BT", color = colorPalette,
#' 	shapeVar = "sex", shape = 15:16,
#' 	sizeVar = "age", sizeRange = c(2, 6),
#' 	symmetryAxes = "separate",
#' 	topSamples = 15, topSamplesVar = "cod", topSamplesColor = "black",
#' 	topSamplesJust = c(1, 0), topSamplesCex = 3)
#' )
#' @author Laure Cougnaud
#' @import Biobase
#' @importFrom utils capture.output
#' @importFrom Rtsne Rtsne
#' @export
esetTsne <- function(eset, 
	psids = 1:nrow(eset),
	trace = TRUE,
	colorVar = character(), 
	color = if(length(colorVar) == 0)	"black"	else	character(),
	shapeVar = character(), 
	shape = if(length(shapeVar) == 0)	15	else	numeric(),
	sizeVar = character(), 
	size = if(length(sizeVar) == 0){
		ifelse(typePlot[1] == "interactive" && packageInteractivity[1] == "plotly", 20, 2.5
		)
	}else{numeric()},
	sizeRange = numeric(),
	alphaVar = character(), 
	alpha = if(length(alphaVar) == 0)	1	else	numeric(), 
	alphaRange = numeric(),
	title = "", 
	#parameters for tsne
	Rtsne.args = list(perplexity = floor((ncol(eset)-1)/3),
		theta = 0.5, dims = 2, initial_dims = 50),
	fctTransformDataForInputTsne = NULL,
	symmetryAxes = c("combine", "separate", "none"),
	packageTextLabel = c("ggrepel", "ggplot2"),
	topSamples = 10,
	topSamplesCex = ifelse(
		typePlot[1] == "interactive" && packageInteractivity[1] == "plotly",
		10, 2.5),
	topSamplesVar = character(),
	topSamplesJust = c(0.5, 0.5), topSamplesColor = "black",
	includeLegend = TRUE, includeLineOrigin = TRUE,
	typePlot = c("static", "interactive"), 
	packageInteractivity = c("plotly", "ggvis"),
	figInteractiveSize  = c(600, 400), ggvisAdjustLegend = TRUE, 
	interactiveTooltip = TRUE, interactiveTooltipExtraVars = character(),
	returnAnalysis = FALSE, returnEsetPlot = FALSE){

	if(identical(packageInteractivity, "rbokeh")){
		.Deprecated(msg = paste("The 'rbokeh' interactive plot is deprecated",
			"(as the rbokeh package is archived), a 'plotly' interactive plot",
			"is created instead."
		))
		packageInteractivity <- "plotly"
	}

	symmetryAxes <- match.arg(symmetryAxes)
	packageInteractivity <- match.arg(packageInteractivity)
	packageTextLabel <- match.arg(packageTextLabel)
	
	# get methods depending on the class of the object
	esetMethods <- getMethodsInputObjectEsetVis(eset)

	# to have reproducible results
	set.seed(123)
	
	# criteria on perplexity in Rtsne:
	#nrow(dat) - 1 < 3 * perplexity
	
	symmetryAxes <- match.arg(symmetryAxes)
	
	if(length(psids) <= 1)
		stop(paste("At least two genes should be used for the visualization.",
			"Please change the 'psids' argument accordingly."))
	
	## Extract exprsMat with specified psids
	esetUsed <- eset[psids, ]
	
	## Prepare data for tsne
	inputTsne <- if(!is.null(fctTransformDataForInputTsne))	
		fctTransformDataForInputTsne(esetMethods$exprs(esetUsed))	else	t(esetMethods$exprs(esetUsed))
	#as.matrix((1 - cor(exprs(esetUsed)))/2)# or as.dist((1 - cor(dat))/2)
	argsRtsne <- c(Rtsne.args, list(X = inputTsne))
	
	## Run tsne
	wrapperTnse <- function()	do.call("Rtsne", argsRtsne)
	if(trace)	tsneOut <- wrapperTnse()	else
		outputMessages <- capture.output(tsneOut <- wrapperTnse())
	
	## Extract data for final plot
	dataPlotSamples <- data.frame(tsneOut$Y, esetMethods$sampleNames(esetUsed))
	colnames(dataPlotSamples) <- c("X", "Y", "sampleName")
	rownames(dataPlotSamples) <- as.character(dataPlotSamples$sampleName)
	
	plot <- esetVis::esetPlotWrapper(
		dataPlotSamples = dataPlotSamples, 
		esetUsed = esetUsed, 
		colorVar = colorVar, color = color,
		shapeVar = shapeVar, shape = shape,
		sizeVar = sizeVar, size = size, sizeRange = sizeRange,
		alphaVar = alphaVar, alpha = alpha, alphaRange = alphaRange,
		title = title, symmetryAxes = symmetryAxes,
		topSamples = topSamples, topSamplesCex = topSamplesCex, topSamplesVar = topSamplesVar, topSamplesJust = topSamplesJust,
		includeLegend = includeLegend, includeLineOrigin = includeLineOrigin,
		typePlot = typePlot, 
		figInteractiveSize = figInteractiveSize, ggvisAdjustLegend = ggvisAdjustLegend, interactiveTooltip = interactiveTooltip, interactiveTooltipExtraVars = interactiveTooltipExtraVars,
		xlab = "X", ylab = "Y",
		returnTopElements = returnAnalysis,
		packageInteractivity = packageInteractivity,
		packageTextLabel = packageTextLabel,
		returnEsetPlot = returnEsetPlot)
	
	res <- if(!returnAnalysis)	plot else
		c(
			list(
				analysis = list(
					dataPlotSamples = dataPlotSamples,
					esetUsed = esetUsed
				)
			),
			if(!is.null(plot$topElements))	list(topElements = plot$topElements),
			list(plot = if(inherits(plot, "ggplot"))	plot	else	plot$plot)
		)

	return(res)

}

---
title: "Vignette of the esetVis package"
author: "Laure Cougnaud"
date: "`r Sys.Date()`"
output: 
  rmarkdown::html_document:
    css: custom.css
    toc: true
    toc_float:
      collapsed: true
    number_sections: true
references:
- id: spm
  title: Spectral mapping, a technique for classifying biological activity profiles of chemical compounds
  author:
  - family: P.J.
    given: Lewi
  type: article-journal
  volume: 26
  page: 1295-1300
  publisher: Arzneimittel Forschung (Drug Research)
  issued:
    year: 1976
- id: tsne
  title: Visualizing High-Dimensional Data Using t-SNE
  author:
  - family: van der Maaten
    given: L.J.P.
  type: article-journal
  volume: 26
  page: 2579-2605
  publisher: Journal of Machine Learning Research
  issued:
    year: 2008
- id: ldaFisher
  title: The Use of Multiple Measurements in Taxonomic Problems
  author:
  - family: Fisher
    given: R. A.
  type: article-journal
  volume: 7 (2)
  page: 179-188
  publisher: Annals of Eugenics
  issued:
    year: 1936
vignette: >
  %\VignetteIndexEntry{esetVis package}
  %\VignetteEngine{knitr::rmarkdown}
  \usepackage[utf8]{inputenc}
---

# Introduction

This document explains the functionalities available in the
**esetVis** package.


This package contains wrapper functions for three types of visualization:
**spectral map**[@spm], **tsne**[@tsne] and **linear discriminant
analysis**[@ldaFisher] for data contained in a _expressionSet
Bioconductor_  (or an _SummarizedExperiment_) object. The visualizations are
available in two types: **static**, via the
[ggplot2](https://cran.r-project.org/package=ggplot2)
package or **interactive**, via the
[ggvis](https://cran.r-project.org/package=ggvis)
or [plotly](https://cran.r-project.org/package=plotly)
packages.

```{r options, echo = FALSE}
	
	library(knitr)
	opts_chunk$set(echo = TRUE, results = 'asis', warning = FALSE, 
		error = FALSE, message = FALSE, cache = FALSE,
		fig.width = 8, fig.height = 7,
		#out.width = '0.7\\textwidth', 
		fig.align = 'center')#, out.height = 0.5\\textwidth', fig.path = 'graphs/') 
	options(width = 170)#, stringsAsFactors = FALSE
	options(warn = 1)#instead of warn = 0 by default -> to have place where warnings occur in the call to Sweave function
	
```

```{r loadLibraries, results = 'hide', echo = FALSE}

	library(esetVis)
#	library(xtable)
	library(pander)

```

# Example dataset

## ExpressionSet object

The _ALL_ dataset contains microarray results from 128 patients with
acute lymphoblastic leukemia (ALL). The data is contained in a
_Bioconductor_ `ExpressionSet` object.  Extra gene annotation is
added to the object, via the annotation package `hgu95av2.db`.

```{r ALL-loadAndFormatAllDataset}

	library(ALL)
	data(ALL)
	
	# to get gene annotation from probe IDs (from the paper HGU95aV2 gene chip was used)
	library("hgu95av2.db")
	library("AnnotationDbi")
	probeIDs <- featureNames(ALL)
	geneInfo <- AnnotationDbi::select(hgu95av2.db, probeIDs, 
		c("ENTREZID", "SYMBOL", "GENENAME"), "PROBEID")
	# 482 on the 12625 probe IDs don't have ENTREZ ID/SYMBOL/GENENAME

	# remove genes with duplicated annotation: 1214
	geneInfoWthtDuplicates <- geneInfo[!duplicated(geneInfo$PROBEID), ]

	# remove genes without annotation: 482
	genesWthtAnnotation <- rowSums(is.na(geneInfoWthtDuplicates)) > 0
	geneInfoWthtDuplicatesAndWithAnnotation <- geneInfoWthtDuplicates[!genesWthtAnnotation, ]
	
	probeIDsWithAnnotation <- featureNames(ALL)[featureNames(ALL) %in% 
		geneInfoWthtDuplicatesAndWithAnnotation$PROBEID]
	ALL <- ALL[probeIDsWithAnnotation, ]
	
	fData <- geneInfoWthtDuplicatesAndWithAnnotation[
		match(probeIDsWithAnnotation, geneInfoWthtDuplicatesAndWithAnnotation$PROBEID), ]
	rownames(fData) <- probeIDsWithAnnotation
	fData(ALL) <- fData

	# grouping variable: B = B-cell, T = T-cell
	groupingVariable <- pData(ALL)$BT
	
	# create custom palette
	colorPalette <- c("dodgerblue", 
		colorRampPalette(c("white","dodgerblue2", "darkblue"))(5)[-1], 
		"red", colorRampPalette(c("white", "red3", "darkred"))(5)[-1])
	names(colorPalette) <- levels(groupingVariable)
	color <- groupingVariable; levels(color) <- colorPalette
	
	# reformat type of the remission
	remissionType <- ifelse(is.na(ALL$remission), "unknown", as.character(ALL$remission))
	ALL$remissionType <- factor(remissionType,
		levels = c("unknown", "CR", "REF"), 
		labels = c("unknown", "achieved", "refractory"))

```

```{r ALL-loadAndFormatAllDataset-extraNotes, echo = FALSE}

	fvarMetadata(ALL)$labelDescription <- paste(rownames(fvarMetadata(ALL)),
		"obtained from the Bioconductor hgu95av2.db package")	

```

```{r rmAnnotObjectsFromMemory, echo = FALSE}

	rm(list = c("fData", ls(pattern = "^(geneInfo|probeIDs)")));tmp <- gc(verbose = FALSE)
	#for testing: sort(sapply(ls(), function(x) object.size(get(x))))

```

Following tables detail some sample and gene annotation contained in the _ALL_
`ExpressionSet` used in the vignette.

```{r ALL-someDataCharacteristics, echo = FALSE}

	varLabelsUsed <- c("cod", "sex", "age", "BT", "remissionType")
	pander(head(pData(ALL)[, varLabelsUsed]),
		caption = "subset of the **phenoData** of the ALL dataset for the first genes")
	
	pander(head(fData(ALL)), 
        caption = "**featureData** of the ALL dataset for the first genes")

```

## SummarizedExperiment object

The functions of the package also supports object
of class:
[`SummarizedExperiment`](http://bioconductor.org/packages/release/bioc/html/SummarizedExperiment.html).
Note: In this case, the functions `fData`, `pData`, `exprs` should be replaced
by their corresponding functions `rowData`, `colData` and `assay`.

# Spectral map: `esetSpectralMap`

## Simple spectral map

The function `esetSpectralMap` creates a **spectral
map**[@spm] for the dataset. Some default parameters are set, e.g.
to print the top 10 samples and top 10 genes, to display the first two
dimensions of the analysis...

The resulting biplot contains two components:

* in the background, a cloud of the genes coordinates (plotted with the
  [hexbin](https://cran.r-project.org/web/packages/hexbin/index.html) package)
* in the foreground, each sample of the data is represented as a single
  point/symbol

Here is an example for the _ALL_ dataset, with the default parameters.

```{r ALL-esetSimple}

	print(esetSpectralMap(eset = ALL))

```

## Additional sample information

Several annotation variables are available in the `eSet`.

### General

Four different aesthetics `[aes]` can be used to display these variables:

* **color**, with the tag `color`
* **transparency**, with the tag `alpha`
* **size**, with the tag `size`
* **shape**, with the tag `shape`

For each of this aesthetic `[aes]`, several parameters are available:

* `[aes]Var`: name of the column of the `phenoData` of the
  `eSet` used for the aesthetic, i.e. `colorVar`
* `[aes]`: palette/specified shape/size used for the aesthetic, i.e.
  `color`

### Custom size and transparency

Custom size and the transparency (variables `sizeVar` and `alphaVar`) can be
specified:

* if the size/transparency is a numerical
variable (numeric or integer), the range of the size/transparency can be
specified respectively with the arguments `sizeRange` and
`alphaRange`
* in the other cases (factor or character), custom
size/transparency can be specified directly respectively via the `size`
and `alpha`   arguments

In the example, the type and stage of the disease (variable
_BT_) is used for coloring, the remission type for the transparency, the
_sex_ for the shape and the _age_ for the size of the points. A custom color
palette is specified via the `color` argument.

```{r ALL-esetSampleAnnotation1}

	print(esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Sample annotation (1)",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex", 
		sizeVar = "age", sizeRange = c(0.1, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamples = 0, topGenes = 0, cloudGenes = TRUE))

```

Just for the demonstration, another visualization of the same dataset,
using this time a continuous variable: _age_ for coloring and transparency, a
factor for the size and _BT_ for the shape.
Because the output is a `ggplot` object, any additional customization
not implemented via specific parameters, the plot can be modified
with additional `ggplot2` functions, e.g. with the color of the gradient.

```{r ALL-esetSampleAnnotation2, fig.height = 8}

	gg <- esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Sample annotation (2)",
		colorVar = "age",
		shapeVar = "BT", shape = 15+1:nlevels(ALL$BT),
		sizeVar = "remissionType", size = c(2, 4, 6),
		alphaVar = "age", alphaRange = c(0.2, 0.8),
		topSamples = 0, topGenes = 0, cloudGenes = TRUE)
	# change color of gradient
	library(ggplot2)
	gg <- gg + scale_color_gradientn(colours = 
		colorRampPalette(c("darkgreen", "gold"))(2)
	)
	print(gg)

```

## Custom gene representation

Several parameters related to the gene visualization are available:

* gene subset: only a subset of the genes (at least two) can be displayed, via
  the argument `psids`
* gene cloud:
  + inclusion/removal of the gene cloud via `cloudGenes`
  + number of bins specification via `cloudGenesNBins`
  + cloud color via `cloudGenesColor`
  + legend:
    - inclusion/removal of the gene legend via `cloudGenesIncludeLegend`
    - title legend specified via `cloudGenesTitleLegend`
    
The spectral map is done only on the subset of the genes, with the number of
bins, color, and legend title modified.
    
```{r ALL-customGeneRepresentation}

	print(esetSpectralMap(eset = ALL,
		psids = 1:1000,
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Custom cloud genes",
		topSamples = 0, topGenes = 0, 
		cloudGenes = TRUE, cloudGenesColor = "red", cloudGenesNBins = 50,
		cloudGenesIncludeLegend = TRUE, cloudGenesTitleLegend = "number of features"))

```

## Label outlying elements

###  Parameters

Three different kind of elements can be labelled in the
plot: **genes, samples and gene sets**.

For each `[element]`, several parameters are available:

* `top[Elements]`: number of elements to label
* `top[Elements]Var` (not available for gene sets): column of the
corresponding element in the `eSet` used for labelling, in
`phenoData` for sample and `featureData` for gene. If not
specified, the feature/sample names of the `eSet` are used
* `top[Elements]Just`: label justification
* `top[Elements]Cex`: label size
* `top[Elements]Color`: label color

### Method to select top elements

The **distance (sum of squared coordinates) of the gene/sample/gene set to the
origin of the plot** is used to rank the elements, and to extract the top
'outlying' ones.

### Package used for static plot

By default (and if installed), the package
[ggrepel](https://cran.r-project.org/web/packages/ggrepel/index.htm)
is used for text labelling (as in this vignette), to avoid overlapping labels.
The text labels can also be displayed with the
[ggplot2](https://cran.r-project.org/web/packages/ggplot2/index.html)
by setting the parameter `packageTextLabel` to `ggplot2` (default
`ggrepel`).

### Example

In the example, the top genes are labelled with gene symbols (_SYMBOL_
column of the phenoData of the eSet), and the top samples with patient
identifiers (_cod_ column of the phenoData of the eSet).

```{r ALL-genesSamplesAnnotation}

	print(esetSpectralMap(eset = ALL, 
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Spectral map \n Label outlying samples and genes"),
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topGenes = 10, topGenesVar = "SYMBOL",
		topGenesCex = 2, topGenesColor = "darkgrey",
		topSamples = 15, topSamplesVar = "cod", topSamplesColor = "chocolate4",
		topSamplesCex = 3
	))

```

## Gene sets annotation

Genes can be grouped into biologically meaningful gene sets, which can be
labelled in the biplot.

Compared to previous section, two additional parameters are
available: 

* `geneSets` (**required**): list of
gene sets. Each element in the list should contain genes identifiers, and the
  list should be named
* `geneSetsVar`: column of the featureData of the eSet used to map the gene
  identifiers contained in the `geneSets` object
* `geneSetsMaxNChar`: number of characters used in gene sets labels

The `geneSets` can be created with the `getGeneSetsForPlot`
function (wrapper around the `getGeneSets` function of the
[MLP](https://www.bioconductor.org/packages/release/bioc/html/MLP.html)
package), which can extract gene sets available in the Gene Ontology (Biological
Process, Molecular Function and Cellular Component) and KEGG databases. Custom
gene set lists can also be provided.

In the following example, only the pathways from the _GOBP_ database are
used.

```{r ALL-extractGeneSets}

	geneSets <- getGeneSetsForPlot(
		entrezIdentifiers = fData(ALL)$ENTREZID, 
		species = "Human", 
		geneSetSource = 'GOBP',
		useDescription = TRUE)

```

Then this list of gene sets is provided to the `esetSpectralMap`
function.

```{r ALL-geneSetAnnotation}

	print(esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Gene set annotation",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topGenes = 0,
		topGeneSets = 4, geneSets = geneSets, geneSetsVar = "ENTREZID", geneSetsMaxNChar = 30,
		topGeneSetsCex = 2, topGeneSetsColor = "purple"))

```

Note: because of the inherent hierarchical structure of the Gene Ontology
database, sets of genes can be very similar, which can result in close (even
overlapping) labels in the visualization.

```{r rmGeneSetsFromMemory, echo = FALSE}

	rm(geneSets);tmp <- gc(verbose = FALSE)

```

## Dimensions of the biplot

In all previous plots, only the first dimensions of the principal component
analysis were visualized, this can be specified via the `dim` parameter.
The third and fourth dimensions are visualized in the next Figure. This
parameter is only available for the spectral map visualization.

```{r ALL-dimensionsBiPlot}

	print(esetSpectralMap(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Spectral map \n Dimensions of the PCA",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(0.5, 4),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		dim = c(3, 4)))

```

## Implementation

The function uses the `mpm` and `plot.mpm` function from the
`mpm` package. Some default parameters are set for these two functions,
they can be changed via the `mpm.args` and `plot.mpm.args`
arguments. For further details, refer to the documentation of these two
functions.

Note: One important argument is `logtrans` in `mpm`
function, which indicates if the **data should be first log-transformed**
before the computation. It is set by default to FALSE, **assuming that the
data are already in the log scale** (it is the case for the _ALL_
dataset).

## Interactive spectral map

All plots available in the `esetVis` package can be **static or
interactive**.

The argument `typePlot` can be set respectively to
`static` (by default), in this case `ggplot2` is used, or
`interactive`, in this case either the `ggvis` or
`plotly` package is used.

Two functionalities of the interactive plot can be of
interest for such high-dimensional data:

* **hover** to check sample annotation.
  By default, only the sample variables used for the aesthetics are displayed
  when hovering on a specific sample dot. Additional sample variables
  (contained in `phenoData`) displayed in the hover can be specified via the
  `interactiveTooltipExtraVars` parameter.
* **zoom** to focus on specific sample in high-dimensional
  dataset

An interactive version of the spectral map of the previous section is created.

### plotly

```{r ALL-interactivePlot-plotly}
esetSpectralMap(
	eset = ALL,
	title = paste("Acute lymphoblastic leukemia dataset - spectral map"),
	colorVar = "BT", color = unname(colorPalette),
	shapeVar = "sex",
	alphaVar = "remissionType",
	typePlot = "interactive", packageInteractivity = "plotly",
	figInteractiveSize = c(700, 600),
	topGenes = 10, topGenesVar = "SYMBOL",
	topSamples = 10,
	# use all sample variables for hover
	interactiveTooltipExtraVars = varLabels(ALL)
)
```

### ggvis

Note: as `ggvis` plot requires to have a R session running, only a static
version of the plot is included.

```{r ALL-interactivePlot-ggvis, cache = FALSE}
library(ggvis)

# embed a static version of the plot
esetSpectralMap(
	eset = ALL,
	title = paste("Acute lymphoblastic leukemia dataset - spectral map"),
	colorVar = "BT", color = unname(colorPalette),
	shapeVar = "sex",
	alphaVar = "remissionType",
	typePlot = "interactive", packageInteractivity = "ggvis",
	figInteractiveSize = c(700, 600),
	sizeVar = "age", sizeRange = c(2, 6),
	# use all phenoData variables for hover
	interactiveTooltipExtraVars = varLabels(ALL)
)
```

# Tsne: `esetTsne`

## General

Another unsupervised visualization is available in the package:
**t-Distributed Stochastic Neighbor Embedding** (_tsne_). The
function `esetTnse` uses the `Rtsne` function of the same
package.

Most of the previous parameters discussed for the spectral map are
available for this visualization, at the exception of:

* parameter linked to gene annotation/labelling. The gene annotation is  not
  (yet) mapped to the sample coordinated obtained as output from the
  `Rtsne` function
* parameter specific to the spectral map, i.e. `dim`

Arguments to the `Rtsne` function can be specified via the
`Rtsne.args` argument.

Here is an example of tsne, for the same dataset and same annotation/labelling.

<!-- takes 8s  -->
```{r ALL-tsne}

	print(esetTsne(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Tsne",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod"
	))

```

## Additional pre-processing step

The tsne can be quite time-consuming, especially for big data. As the
`Rtsne` function used in the background can also uses an object of class
`dist`, the data can be pre-transformed before running the `tsne`
analysis. The argument `fctTransformDataForInputTsne` enables to specify
a custom function to pre-transform the data.

```{r ALL-tsne-preProcessing}

	print(esetTsne(eset = ALL, 
		title = "Acute lymphoblastic leukemia dataset \n Tsne",
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod",
		fctTransformDataForInputTsne = 
			function(mat)	as.dist((1 - cor(mat))/2)
	))

```

# Linear discriminant analysis: `esetLda`

Another visualization, this time semi-supervised (as a variable is used for the
computation), is included: **Linear Discriminant Analysis**. This uses the
`lda` function from the `MASS` package.

This function maximizes the variance between levels of a factor, here describing
some sample annotation, specified via the `ldaVar` argument.

As this analysis can be quite time consuming,
for the demonstration, the analysis is run only a random feature subset
of the data. 

## All samples

The `returnAnalysis` parameter can be used, to extract the
analysis which will be used as input for the `esetPlotWrapper` function,
used in the background. It is available also for the `esetSpectralMap`
and `esetTnse` functions.

```{r ALL-Lda, fig.keep = 'first'}

	# extract random features, because analysis is quite time consuming
	retainedFeatures <- sample(featureNames(ALL), size = floor(nrow(ALL)/5))
	
	# run the analysis
	outputEsetLda <- esetLda(eset = ALL[retainedFeatures, ], ldaVar = "BT",
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Linear discriminant analysis on BT variable \n",
			"(Subset of the feature data)"),
		colorVar = "BT", color = colorPalette,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod", topGenesVar = "SYMBOL",
		returnAnalysis = TRUE
	)

	# extract and print the ggplot object
	print(outputEsetLda$plot)
```

The top elements (here genes and samples) labelled in the plot can be accessed
via the `topElements` slot of the object. These are labelled with the
identifiers used in the plot and named with sample/gene identifiers of the
`eset`.

```{r ALL-Lda-topElements}	

	# extract top elements labelled in the plot
	pander(t(data.frame(topGenes = outputEsetLda$topElements[["topGenes"]])))
	pander(t(data.frame(topSamples = outputEsetLda$topElements[["topSamples"]])))

```

When `returnAnalysis` is set to TRUE, the output of the function can be
used as input to the `esetPlotWrapper` function, and extra parameters can
then be modified.

Here the variable used for the shape of the points for the samples is changed to
type of remission (_remissionType_ column).

```{r ALL-Lda-changeSomeParameters}

	# to change some plot parameters
	esetPlotWrapper(
		dataPlotSamples = outputEsetLda$analysis$dataPlotSamples,
		dataPlotGenes = outputEsetLda$analysis$dataPlotGenes,
		esetUsed = outputEsetLda$analysis$esetUsed,
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Linear discriminant analysis on BT variable (2) \n",
			"(Subset of the feature data)"),
		colorVar = "BT", color = colorPalette,
		shapeVar = "remissionType", 
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod", topGenesVar = "SYMBOL"
	)

```

```{r rmOutputEsetLdaFromMemory, echo = FALSE}

	rm(outputEsetLda);tmp <- gc(verbose = FALSE)

```

## Data sample subset

From the previous visualization (obtained on a subset of the genes), the biggest
difference between all levels of the type/stage of the disease seems to reside
between all B-cells (tagged _B_) and T-cells (tagged _T_). It might be
interesting to focus on a subset of the data, e.g. only one cell type.

```{r ALL-Lda-BcellOnly}

	# keep only 'B-cell' samples
	ALLBCell <- ALL[, grep("^B", ALL$BT)]
	ALLBCell$BT <- factor(ALLBCell$BT)
	colorPaletteBCell <- colorPalette[1:nlevels(ALLBCell$BT )]
	
	print(esetLda(eset = ALLBCell[retainedFeatures, ], ldaVar = "BT",
		title = paste("Acute lymphoblastic leukemia dataset \n",
			"Linear discriminant analysis on BT variable \n B-cell only",
			"(Subset of the feature data)"
		),
		colorVar = "BT", color = colorPaletteBCell,
		shapeVar = "sex",
		sizeVar = "age", sizeRange = c(2, 6),
		alphaVar = "remissionType", alpha = c(0.3, 0.6, 0.9),
		topSamplesVar = "cod", topGenesVar = "SYMBOL",
	))

```

```{r rmALLBCellFromMemory, echo = FALSE}

	rm(ALLBCell);tmp <- gc(verbose = FALSE)

```
The subcell type which seems to differ the most within all B-cell type is
the
first one: _B1_ (most of these samples at the right side of the plot).

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEset-plotly.R
\name{plotlyPlotEset}
\alias{plotlyPlotEset}
\title{visualize and \link{esetPlot} with the the 'plotly' package}
\usage{
plotlyPlotEset(object)
}
\arguments{
\item{object}{object of class \link{esetPlot}}
}
\value{
\code{plotly} plot
}
\description{
visualize and \link{esetPlot} with the the 'plotly' package
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetPlot-class.R
\docType{class}
\name{ggplotEsetPlot-class}
\alias{ggplotEsetPlot-class}
\alias{ggplotEsetPlot}
\title{a S4 class to represent \code{ggplot} plots}
\value{
S4 object of class \code{ggplotEsetPlot}
}
\description{
a S4 class to represent \code{ggplot} plots
}
\section{Slots}{

\describe{
\item{\code{returnTopElements}}{logical, if TRUE (FALSE by default) 
return the outlying elements labelled in the plot (if any)}

\item{\code{title}}{string or expression with plot title, '' by default}

\item{\code{xlab}}{string or expression with label for the x axis}

\item{\code{ylab}}{string or expression with label for the y axis}
}}

\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{setFixElement}
\alias{setFixElement}
\title{check if the aesthetic is fixed (e.g. color, shape, size 'palette')}
\usage{
setFixElement(typeVar, valVar)
}
\arguments{
\item{typeVar}{name of variable for aesthetic}

\item{valVar}{fixed value of variable of aesthetic}
}
\value{
logical, if TRUE the element is fixed
}
\description{
check if the aesthetic is fixed (e.g. color, shape, size 'palette')
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetLda.R
\name{esetLda}
\alias{esetLda}
\title{plot a biplot of a linear discriminant analysis of an \linkS4class{eSet} object}
\usage{
esetLda(
  eset,
  ldaVar,
  psids = 1:nrow(eset),
  dim = c(1, 2),
  colorVar = character(),
  color = if (length(colorVar) == 0) "black" else character(),
  shapeVar = character(),
  shape = if (length(shapeVar) == 0) 15 else numeric(),
  sizeVar = character(),
  size = if (length(sizeVar) == 0) {
     ifelse(typePlot[1] == "interactive" &&
    packageInteractivity[1] == "plotly", 20, 2.5)
 } else {
     numeric()
 },
  sizeRange = numeric(),
  alphaVar = character(),
  alpha = if (length(alphaVar) == 0) 1 else numeric(),
  alphaRange = numeric(),
  title = "",
  symmetryAxes = c("combine", "separate", "none"),
  packageTextLabel = c("ggrepel", "ggplot2"),
  cloudGenes = TRUE,
  cloudGenesColor = "black",
  cloudGenesNBins = sqrt(length(psids)),
  cloudGenesIncludeLegend = FALSE,
  cloudGenesTitleLegend = "nGenes",
  topGenes = 10,
  topGenesCex = ifelse(typePlot[1] == "interactive" && packageInteractivity[1] ==
    "plotly", 10, 2.5),
  topGenesVar = character(),
  topGenesJust = c(0.5, 0.5),
  topGenesColor = "black",
  topSamples = 10,
  topSamplesCex = ifelse(typePlot[1] == "interactive" && packageInteractivity[1] ==
    "plotly", 10, 2.5),
  topSamplesVar = character(),
  topSamplesJust = c(0.5, 0.5),
  topSamplesColor = "black",
  geneSets = list(),
  geneSetsVar = character(),
  geneSetsMaxNChar = numeric(),
  topGeneSets = 10,
  topGeneSetsCex = ifelse(typePlot[1] == "interactive" && packageInteractivity[1] ==
    "plotly", 10, 2.5),
  topGeneSetsJust = c(0.5, 0.5),
  topGeneSetsColor = "black",
  includeLegend = TRUE,
  includeLineOrigin = TRUE,
  typePlot = c("static", "interactive"),
  packageInteractivity = c("plotly", "ggvis"),
  figInteractiveSize = c(600, 400),
  ggvisAdjustLegend = TRUE,
  interactiveTooltip = TRUE,
  interactiveTooltipExtraVars = character(),
  returnAnalysis = FALSE,
  returnEsetPlot = FALSE
)
}
\arguments{
\item{eset}{expressionSet (or SummarizedExperiment) object with data}

\item{ldaVar}{name of variable (in varLabels of the \code{eset}) used for grouping for lda}

\item{psids}{featureNames of genes to include in the plot, all by default}

\item{dim}{dimensions of the analysis to represent, first two dimensions by default}

\item{colorVar}{name of variable (in varLabels of the \code{eset}) used 
for coloring, empty by default}

\item{color}{character or factor with specified color(s) for the points, 
replicated if needed. This is used only if \code{colorVar} is empty.
By default: 'black' if \code{colorVar} is not specified and default 
\code{ggplot} palette otherwise}

\item{shapeVar}{name of variable (in varLabels of the \code{eset}) 
used for the shape, empty by default}

\item{shape}{character or factor with specified shape(s) (pch) for the points, 
replicated if needed. This is used only if \code{shapeVar} is empty.
By default: '15' (filled square) if \code{shapeVar} is not specified 
and default \code{ggplot} shape(s) otherwise}

\item{sizeVar}{name of variable (in varLabels of the \code{eset}) 
used for the size, empty by default}

\item{size}{character or factor with specified size(s) (cex) for the points, 
replicated if needed.
This is used only if \code{sizeVar} is empty.
By default: '2.5' if \code{sizeVar} is not specified (20 for a plotly plot)
and default \code{ggplot} size(s) otherwise}

\item{sizeRange}{size (cex) range used in the plot, possible only 
if the \code{sizeVar} is 'numeric' or 'integer'}

\item{alphaVar}{name of variable (in varLabels of the \code{eset}) 
used for the transparency, empty by default.
This parameter is currently only available for static plot and ggvis (only numeric in this case).}

\item{alpha}{character or factor with specified transparency(s) for the points,
replicated if needed. This is used only if \code{shapeVar} is empty. 
By default: '1' if \code{alphaVar} is not specified and default 
\code{ggplot} alpha otherwise
This parameter is currently only available for static and ggvis.}

\item{alphaRange}{transparency (alpha) range used in the plot, 
possible only if the \code{alphaVar} is 'numeric' or 'integer'
This parameter is currently only available for static and ggvis plot.}

\item{title}{plot title, '' by default}

\item{symmetryAxes}{set symmetry for axes, either:
\itemize{
 \item{'combine' (by default): }{both axes are symmetric and with the same limits}
 \item{'separate': }{each axis is symmetric and has its own limits}
 \item{'none': }{axes by default (plot limits)}
}}

\item{packageTextLabel}{package used to label the outlying genes/samples/gene sets,
either \code{ggrepel} (by default, only used if package \code{ggrepel} is available),
or \code{ggplot2}}

\item{cloudGenes}{logical, if TRUE (by default), include the 
cloud of genes in the plot}

\item{cloudGenesColor}{if \code{cloudGenes} is TRUE, 
color for the cloud of genes, black by default}

\item{cloudGenesNBins}{number of bins to used for the clouds of genes,
by default the square root of the number of genes}

\item{cloudGenesIncludeLegend}{logical, if TRUE (FALSE by default) 
include the legend for the cloud of genes (in the top position if multiple legends)}

\item{cloudGenesTitleLegend}{string with title for the legend for the cloud of genes
'nGenes' by default}

\item{topGenes}{numeric indicating which percentile (if <1) or number (if >=1) of genes
most distant to the origin of the plot to annotate, by default: 10 genes are selected
If no genes should be annotated, set this parameter to 0
Currently only available for static plot.}

\item{topGenesCex}{cex for gene annotation (used when \code{topGenes} > 0)}

\item{topGenesVar}{variable of the featureData used to label the genes, 
by default: empty, the featureNames are used for labelling (used when \code{topGenes} > 0)}

\item{topGenesJust}{text justification for the genes 
(used when \code{topGenes} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
by default: c(0.5, 0.5) so centered}

\item{topGenesColor}{text color for the genes 
(used when \code{topGenes} > 0), black by default}

\item{topSamples}{numeric indicating which percentile (if <1) or number (if >=1) 
of samples most distant to the origin of the plot to annotate, 
by default: 10 samples are selected
If no samples should be annotated, set this parameter to 0.
Currently available for static plot.}

\item{topSamplesCex}{cex for sample annotation (used when \code{topSamples} > 0)}

\item{topSamplesVar}{variable of the phenoData used to label the samples, 
by default: empty, the sampleNames are used for labelling 
(used when \code{topSample}s > 0)}

\item{topSamplesJust}{text justification for the samples 
(used when \code{topSamples} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
by default: c(0.5, 0.5) so centered}

\item{topSamplesColor}{text color for the samples 
(used when \code{topSamples} > 0), black by default}

\item{geneSets}{list of gene sets/pathways, each containing 
identifiers of genes contained in the set.
E.g. pathways from Gene Ontology databases output from the 
\code{\link{getGeneSetsForPlot}} function or any custom list of pathways.
The genes identifiers should correspond to the variable 
\code{geneSetsVar} contained in the phenoData, if not specified
the featureNames are used.
If several gene sets have the same name, they will be 
combine to extract the top gene sets.}

\item{geneSetsVar}{variable of the featureData used to 
match the genes contained in geneSets,
most probably ENTREZID, if not specified the featureNames
 of the eSet are used
Only used when \code{topGeneSets} > 0 and the parameter 
geneSets is specified.}

\item{geneSetsMaxNChar}{maximum number of characters for pathway names, 
by default keep entire names
Only used when \code{topGeneSets} > 0 and the parameter \code{geneSets} is specified.
If \code{returnAnalysis} is set to TRUE and \code{geneSetsMaxNChar} specified, 
the top pathways will be returned in the output object, named with the identifiers used in the plot 
(so with maximum \code{geneSetsMaxNChar} number of characters)}

\item{topGeneSets}{numeric indicating which percentile (if <=1) or number (if >1) of gene sets
most distant to the origin of the plot to annotate, by default: 10 gene sets are selected
If no gene sets should be annotated, set this parameter to 0.
Currently available for static plot.
Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.}

\item{topGeneSetsCex}{cex for gene sets annotation
Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.}

\item{topGeneSetsJust}{text justification for the gene sets 
by default: c(0.5, 0.5) so centered
Only used when \code{topGeneSets} > 0, the parameter \code{geneSets} 
is specified and if \code{packageTextLabel} is \code{ggplot2}.}

\item{topGeneSetsColor}{color for the gene sets 
(used when \code{topGeneSets} > 0 and \code{geneSets} is specified), black by default
Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.}

\item{includeLegend}{logical if TRUE (by default) include a legend, 
otherwise not}

\item{includeLineOrigin}{if TRUE (by default) include vertical line at 
x = 0 and horizontal line at y = 0}

\item{typePlot}{type of the plot returned, either 'static' (static) or 
interactive' (potentially interactive)}

\item{packageInteractivity}{if \code{typePlot} is 'interactive', 
package used for interactive plot,
either 'plotly' (by default) (by default) or 'ggvis'.}

\item{figInteractiveSize}{vector containing the size of the interactive plot, 
as [width, height]
by default: c(600, 400). This is passed to the \code{width} and 
\code{height} parameters of:
\itemize{
 \item{for plotly plots: }{the \code{\link[plotly]{ggplotly}} function}
 \item{for ggvis plots: }{the \code{ggvis::set_options} function}
}}

\item{ggvisAdjustLegend}{logical, if TRUE (by default) adjust the legends in \code{ggvis} to avoid
overlapping legends when multiple legends}

\item{interactiveTooltip}{logical, if TRUE, add hoover functionality showing
sample annotation (variables used in the plot) in the plot}

\item{interactiveTooltipExtraVars}{name of extra variable(s) 
(in varLabels of the \code{eset}) to add in plotlyEsetPlot to label the samples,
empty by default}

\item{returnAnalysis}{logical, if TRUE (FALSE by default), return also the output of the analysis,
and the outlying samples in the topElements element if any, otherwise only the plot object}

\item{returnEsetPlot}{logical, if TRUE return also the \link{esetPlot} object}
}
\value{
if \code{returnAnalysis} is TRUE, return a list:
\itemize{
 \item{analysis: }{output of the spectral map analysis, whose parameters
can be given as input to the \code{\link{esetPlotWrapper}} function}
   \itemize{
		\item{dataPlotSamples: }{coordinates of the samples}
		\item{dataPlotGenes: }{coordinates of the genes}
		\item{esetUsed: }{expressionSet used in the plot}
	  }
  \item{topElements: }{list with top outlying elements if any, possibly genes, samples and gene sets}
  \item{plot: }{the plot output}
}
otherwise return only the plot
}
\description{
\code{esetLda} reduces the dimension of the data contained in the \linkS4class{eSet} via a linear discriminant analysis
on the specified grouping variable with the \code{lda} function and plot the subsequent biplot,
 possibly with sample annotation and gene annotation contained in the eSet.
}
\examples{
# load data
library(ALL)
data(ALL)

# specify several variables in ldaVar (this might take a few minutes to run...)

# sample subsetting: currently cannot deal with missing values
samplesToRemove <- which(apply(pData(ALL)[, c("sex", "BT")], 1, anyNA)) 

# extract random features, because analysis is quite time consuming
retainedFeatures <- sample(featureNames(ALL), size = floor(nrow(ALL)/5))

# create the plot
esetLda(eset = ALL[retainedFeatures, -samplesToRemove], 
  ldaVar = "BT", colorVar = "BT", shapeVar = "sex", sizeVar = "age",
  title = "Linear discriminant analysis on the ALL dataset")
}
\references{
Fisher, R. A. (1936). The Use of Multiple Measurements in
Taxonomic Problems. Annals of Eugenics, 7 (2), 179--188
}
\seealso{
the function used internally: \code{\link[MASS]{lda}}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEset-plotly.R
\name{varToFm}
\alias{varToFm}
\title{Get formula for a specific variable,
to be used in aesthetic specification in \code{\link[plotly]{plot_ly}}.}
\usage{
varToFm(var)
}
\arguments{
\item{var}{Character vector with variable to combine.
Otherwise with the '+' operator.}
}
\value{
\code{\link[stats]{as.formula}}
}
\description{
Get formula for a specific variable,
to be used in aesthetic specification in \code{\link[plotly]{plot_ly}}.
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetPlot-class.R
\docType{class}
\name{esetPlotInteractive-class}
\alias{esetPlotInteractive-class}
\alias{esetPlotInteractive}
\title{a S4 class to represent interactive plots}
\value{
S4 object of class \code{esetPlotInteractive}
}
\description{
a S4 class to represent interactive plots
}
\section{Slots}{

\describe{
\item{\code{includeTooltip}}{logical, if TRUE, add hoover functionality showing
sample annotation (variables used in the plot) in the plot}

\item{\code{tooltipVars}}{name of extra phenotypic variable(s) 
to add in plotlyEsetPlot to label the samples}

\item{\code{sizePlot}}{vector containing the size of the interactive plot, 
as [width, height], by default: c(600, 400).}

\item{\code{title}}{string plot title, '' by default}

\item{\code{xlab}}{string label for the x axis}

\item{\code{ylab}}{string label for the y axis}
}}

\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{formatManualScale}
\alias{formatManualScale}
\title{extend manual scale values if required}
\usage{
formatManualScale(x, valVar, nameVar)
}
\arguments{
\item{x}{data.frame with \code{nameVar}}

\item{valVar}{fixed value of variable of aesthetic}

\item{nameVar}{name of variable for aesthetic}
}
\value{
vector of manual scales
}
\description{
extend manual scale values if required
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{setManualScale}
\alias{setManualScale}
\title{check if manual aesthetic should be set}
\usage{
setManualScale(x, typeVar, valVar)
}
\arguments{
\item{x}{data.frame with \code{typeVar}}

\item{typeVar}{name of variable for aesthetic}

\item{valVar}{fixed value of variable of aesthetic}
}
\value{
logical, if TRUE the manual scale should be set
}
\description{
This is the case only if \code{typeVar} and \code{valVar} are specified,
and if the variable is not numeric or integer (doesn't work with ggplot2)
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{simpleCap}
\alias{simpleCap}
\title{capitalize the first letter of a word}
\usage{
simpleCap(x)
}
\arguments{
\item{x}{string}
}
\value{
string with first letter capitalized
}
\description{
capitalize the first letter of a word
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEset.R
\docType{methods}
\name{getAxesLimits}
\alias{getAxesLimits}
\alias{getAxesLimits,esetPlot-method}
\title{generic for get axes limits}
\usage{
getAxesLimits(object)

\S4method{getAxesLimits}{esetPlot}(object)
}
\arguments{
\item{object}{\link{plotEset} object}
}
\value{
matrix with limits for axes: columns \code{x} and \code{y}
}
\description{
generic for get axes limits
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/miscellaneous.R
\name{formatOutput}
\alias{formatOutput}
\title{format output of \link{plotEset} function}
\usage{
formatOutput(res, object, type, returnEsetPlot)
}
\arguments{
\item{res}{result of specific \link{plotEset} function}

\item{object}{\code{esetPlot} object or extended class}

\item{type}{string type of plot}

\item{returnEsetPlot}{logical, should the object be returned in the output function?}
}
\value{
result
}
\description{
format output of \link{plotEset} function
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotTopElements.R
\name{plotTopElements}
\alias{plotTopElements}
\title{plot top elements for a static plot}
\usage{
plotTopElements(
  packageTextLabel = c("ggrepel", "ggplot2"),
  cex = 1,
  just = c(0.5, 0.5),
  color = "black",
  returnTopElements = FALSE,
  ...
)
}
\arguments{
\item{packageTextLabel}{package used to label the outlying genes/samples/gene sets,
either 'ggrepel' (by default, only used if package \code{ggrepel} is available),
or 'ggplot2'}

\item{cex}{cex of text in the plot}

\item{just}{justification of elements in the plot, only use if \code{packageTextLabel} is 'ggplot2'}

\item{color}{color for the elements in the plot}

\item{returnTopElements}{logical if TRUE (FALSE by default) return the outlying elements}
}
\value{
\itemize{
 \item{if the \code{elements} are present in the data: if \code{returnTopElements} is: }
 \itemize{
   \item{TRUE: }{return a list with two arguments:} 
     \itemize{
      \item{topElements: }{string with top elements labelled in the plot}
      \item{geomText: }{output of \code{geom_text}}
     }
    \item{FALSE: }{only return the output of \code{geom_text}} 
	 }
 \item{if not, return \code{NULL}}
}
}
\description{
This create \code{geom_text} object with top genes/sample/pathways
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEset.R
\docType{methods}
\name{plotEset}
\alias{plotEset}
\alias{plotEset,ggplotEsetPlot-method}
\alias{plotEset,ggvisEsetPlot-method}
\alias{plotEset,plotlyEsetPlot-method}
\title{plot an \link{plotEset} object}
\usage{
plotEset(object, returnEsetPlot = FALSE)

\S4method{plotEset}{ggplotEsetPlot}(object, returnEsetPlot = FALSE)

\S4method{plotEset}{ggvisEsetPlot}(object, returnEsetPlot = FALSE)

\S4method{plotEset}{plotlyEsetPlot}(object, returnEsetPlot = FALSE)
}
\arguments{
\item{object}{object of class \link{esetPlot}}

\item{returnEsetPlot}{logical, 
if TRUE return also the \link{esetPlot} object,
such as can be re-use for future call to \link{plotEset}}
}
\value{
the plot object if \code{returnEsetPlot} is FALSE,
otherwise a list with 'plot': the plot object
 and 'esetPlot': the \link{esetPlot} object
}
\description{
plot an \link{plotEset} object
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getMethodsInputObjectEsetVis.R
\name{getMethodsInputObjectEsetVis}
\alias{getMethodsInputObjectEsetVis}
\title{wrapper to extract useful functions, depending if the object is 
an ExpressionSet or a SummarizedExperiment.}
\usage{
getMethodsInputObjectEsetVis(x)
}
\arguments{
\item{x}{object}
}
\value{
if the object is an ExpressionSet or a SummarizedExperiment, 
returns a list with the functions specific of the class of \code{x},
and equivalent of the ExpressionSet functions: 'sampleNames', 'featureNames',
'fData', 'pData', 'exprs'
\itemize{
	 \item{sampleNames: }{sample names}
  \item{featureNames: }{feature names}
  \item{fData: }{feature annotation}
  \item{pData: }{sample annotation}
  \item{exprs: }{data matrix}
  \item{varLabels: }{sample annotation variables}
  \item{fvarLabels: }{feature annotation variables}
}
}
\description{
This returns an error is \code{x} is not of the correct class.
The package \code{SummarizedExperiment} should be available if \code{x} is of class \code{SummarizedExperiment}.
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetPlot-class.R
\docType{class}
\name{plotlyEsetPlot-class}
\alias{plotlyEsetPlot-class}
\alias{plotlyEsetPlot}
\title{a S4 class to represent \code{plotly} plots}
\value{
S4 object of class \code{plotlyEsetPlot}
}
\description{
a S4 class to represent \code{plotly} plots
}
\section{Slots}{

\describe{
\item{\code{returnTopElements}}{logical, if TRUE (FALSE by default)
return the outlying elements labelled in the plot (if any)}

\item{\code{size}}{specified size(s) (cex) for the points, replicated if
needed, used only if \code{sizeVar} is empty, a factor or character
by default: '20' if \code{sizeVar} is not specified}
}}

\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetPlot-class.R
\docType{class}
\name{esetPlot-class}
\alias{esetPlot-class}
\alias{esetPlot}
\alias{initialize,esetPlot-method}
\title{An S4 class to represent \code{esetPlot} object
expressionSet with visualization data from
dimension-reduction methods}
\usage{
\S4method{initialize}{esetPlot}(.Object, ...)
}
\arguments{
\item{.Object}{\link{esetPlot} object}

\item{...}{additional class arguments}
}
\value{
S4 object of class \code{esetPlot}
}
\description{
Constructor of the \link{esetPlot} class
}
\section{Slots}{

\describe{
\item{\code{dataPlotSamples}}{data.frame with columns 'X', 'Y' with coordinates 
for the samples and with rownames which 
should correspond and be in the same order as the sampleNames of
\code{esetUsed}}

\item{\code{dataPlotGenes}}{data.frame with two columns 'X' and 'Y' with 
coordinates for the genes}

\item{\code{eset}}{expressionSet (or SummarizedExperiment) object with data}

\item{\code{colorVar}}{name of variable (in varLabels of the \code{eset}) used 
for coloring, empty by default}

\item{\code{color}}{character or factor with specified color(s) for the points, 
replicated if needed. This is used only if \code{colorVar} is empty.
By default: 'black' if \code{colorVar} is not specified and default 
\code{ggplot} palette otherwise}

\item{\code{shapeVar}}{name of variable (in varLabels of the \code{eset}) 
used for the shape, empty by default}

\item{\code{shape}}{character or factor with specified shape(s) (pch) for the points, 
replicated if needed. This is used only if \code{shapeVar} is empty.
By default: '15' (filled square) if \code{shapeVar} is not specified 
and default \code{ggplot} shape(s) otherwise}

\item{\code{sizeVar}}{name of variable (in varLabels of the \code{eset}) 
used for the size, empty by default}

\item{\code{size}}{size character or factor with specified size(s) (cex) for the points, 
replicated if needed.
This is used only if \code{sizeVar} is empty.
By default: '2.5' if \code{sizeVar} is not specified and default 
\code{ggplot} size(s) otherwise}

\item{\code{sizeRange,}}{size (cex) range used in the plot, possible only 
if the \code{sizeVar} is 'numeric' or 'integer'}

\item{\code{alphaVar}}{name of variable (in varLabels of the \code{eset}) 
used for the transparency, empty by default.}

\item{\code{alpha}}{alpha character or factor with specified transparency(s) for the points,
replicated if needed. This is used only if \code{shapeVar} is empty. 
By default: '1' if \code{alphaVar} is not specified and default 
\code{ggplot} alpha otherwise.}

\item{\code{alphaRange}}{transparency (alpha) range used in the plot, 
possible only if the \code{alphaVar} is 'numeric' or 'integer'}

\item{\code{symmetryAxes}}{set symmetry for axes, either:
\itemize{
 \item{'combine' (by default): }{both axes are symmetric and with the same limits}
 \item{'separate': }{each axis is symmetric and has its own limits}
 \item{'none': }{axes by default (plot limits)}
}}

\item{\code{cloudGenes}}{logical, if TRUE (by default), include the 
cloud of genes in the spectral map}

\item{\code{cloudGenesColor}}{if \code{cloudGenes} is TRUE, 
color for the cloud of genes, black by default}

\item{\code{cloudGenesNBins}}{number of bins to used for the clouds of genes,
by default the square root of the number of genes}

\item{\code{cloudGenesIncludeLegend}}{logical, if TRUE (FALSE by default) 
include the legend for the cloud of genes (in the top position if multiple legends)}

\item{\code{cloudGenesTitleLegend}}{string with title for the legend for the cloud of genes
'nGenes' by default}

\item{\code{packageTextLabel}}{package used to label the outlying genes/samples/gene sets,
either \code{ggrepel} (by default, only used if package \code{ggrepel} is available),
or \code{ggplot2}}

\item{\code{topGenes}}{numeric indicating which percentile (if <1) or number (if >=1) of genes
most distant to the origin of the plot to annotate, by default: 10 genes are selected
If no genes should be annotated, set this parameter to 0
Currently only available for static plot.}

\item{\code{topGenesCex}}{cex for gene annotation (used when \code{topGenes} > 0)}

\item{\code{topGenesVar}}{variable of the featureData used to label the genes, 
by default: empty, the featureNames are used for labelling (used when \code{topGenes} > 0)}

\item{\code{topGenesJust}}{text justification for the genes 
(used when \code{topGenes} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
by default: c(0.5, 0.5) so centered}

\item{\code{topGenesColor}}{text color for the genes 
(used when \code{topGenes} > 0), black by default}

\item{\code{topSamples}}{numeric indicating which percentile (if <1) or number (if >=1) 
of samples most distant to the origin of the plot to annotate, 
by default: 10 samples are selected
If no samples should be annotated, set this parameter to 0.
Currently available for static plot.}

\item{\code{topSamplesCex}}{cex for sample annotation (used when \code{topSamples} > 0)}

\item{\code{topSamplesVar}}{variable of the phenoData used to label the samples, 
by default: empty, the sampleNames are used for labelling 
(used when \code{topSample}s > 0)}

\item{\code{topSamplesJust}}{text justification for the samples 
(used when \code{topSamples} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
by default: c(0.5, 0.5) so centered}

\item{\code{topSamplesColor}}{text color for the samples 
(used when \code{topSamples} > 0), black by default}

\item{\code{geneSets}}{list of gene sets/pathways, each containing 
identifiers of genes contained in the set.
E.g. pathways from Gene Ontology databases output from the 
\code{\link{getGeneSetsForPlot}} function or any custom list of pathways.
The genes identifiers should correspond to the variable 
\code{geneSetsVar} contained in the phenoData, if not specified
the featureNames are used.
If several gene sets have the same name, they will be 
combine to extract the top gene sets.}

\item{\code{geneSetsVar}}{variable of the featureData used to 
match the genes contained in geneSets,
most probably ENTREZID, if not specified the featureNames
 of the eSet are used
Only used when \code{topGeneSets} > 0 and the parameter 
geneSets is specified.}

\item{\code{geneSetsMaxNChar}}{maximum number of characters for pathway names, 
by default keep entire names
Only used when \code{topGeneSets} > 0 and the parameter \code{geneSets} is specified.}

\item{\code{topGeneSets}}{numeric indicating which percentile (if <=1) or number (if >1) of gene sets
most distant to the origin of the plot to annotate, by default: 10 gene sets are selected
If no gene sets should be annotated, set this parameter to 0.
Currently available for static plot.
Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.}

\item{\code{topGeneSetsCex}}{cex for gene sets annotation
Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.}

\item{\code{topGeneSetsJust}}{text justification for the gene sets 
by default: c(0.5, 0.5) so centered
Only used when \code{topGeneSets} > 0, the parameter \code{geneSets} 
is specified and if \code{packageTextLabel} is \code{ggplot2}.}

\item{\code{topGeneSetsColor}}{color for the gene sets 
(used when \code{topGeneSets} > 0 and \code{geneSets} is specified), black by default
Only used when \code{topGeneSets} > 0 and the parameter geneSets is specified.}

\item{\code{includeLegend}}{logical if TRUE (by default) include a legend, 
otherwise not}

\item{\code{includeLineOrigin}}{if TRUE (by default) include vertical line at 
x = 0 and horizontal line at y = 0}
}}

\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/getGeneSetsForPlot.R
\name{getGeneSetsForPlot}
\alias{getGeneSetsForPlot}
\title{get gene sets for plot of \linkS4class{eSet} object.}
\usage{
getGeneSetsForPlot(
  entrezIdentifiers,
  species = "Human",
  geneSetSource = c("GOBP", "GOMF", "GOCC", "KEGG"),
  useDescription = TRUE,
  trace = TRUE
)
}
\arguments{
\item{entrezIdentifiers}{string with Entrez Gene identifiers of the genes of interest}

\item{species}{species to use, given to the \code{\link[MLP]{getGeneSets}} function}

\item{geneSetSource}{gene set source, either 'GOBP', 'GOMF', 'GOCC' or 'KEGG'.
Multiple choices are available}

\item{useDescription}{logical, if TRUE (by default) use the description to 
label the gene sets, otherwise use the original gene set identifiers
Function 'substr' is used.}

\item{trace}{logical, if TRUE (by default) 
a few extra information are printed during the process}
}
\value{
list with gene sets, each element is a gene set and 
contains the ENTREZ IDs of the genes contained in this set.
If \code{useDescription} is:
\itemize{
	\item{FALSE: }{pathways are labelled with identifiers (Gene Ontology IDs for GOBP, GOMF and GOCC, KEGG IDs for KEGG)}
 \item{TRUE: }{pathways are labelled with gene sets descriptions}
}
}
\description{
get and format gene sets to be used as \code{geneSets} for the functions:
\code{\link{esetSpectralMap}}, \code{\link{esetLda}}, or \code{\link{esetPlotWrapper}}
Use the \code{\link[MLP]{getGeneSets}} function to get the gene sets,
combine all databases, and format the gene sets name if required.
}
\examples{
# example dataset
library(ALL)
data(ALL)

# get gene annotation from probe IDs
library("hgu95av2.db")
probeIDs <- featureNames(ALL)
geneInfo <- select(hgu95av2.db, probeIDs,"ENTREZID", "PROBEID")

# get pathway annotation for the genes contained in the ALL dataset (can take a few minutes)
geneSets <- getGeneSetsForPlot(entrezIdentifiers = geneInfo$ENTREZID, species = "Human", 
	geneSetSource = 'GOBP',
	useDescription = FALSE, trace = TRUE)
head(geneSets) # returns a pathway list of genes

# gene sets labelled with gene sets description
geneSets <- getGeneSetsForPlot(entrezIdentifiers = geneInfo$ENTREZID, species = "Human", 
	geneSetSource = 'GOBP', useDescription = TRUE, trace = TRUE)
head(geneSets) # returns a pathway list of genes

# see also vignette for an example of the use of this function as input for the esetSpectralMap, esetLda or esetPlotWrapper functions
}
\seealso{
the function used internally: \code{\link[MLP]{getGeneSets}}
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetTsne.R
\name{esetTsne}
\alias{esetTsne}
\title{plot a t-SNE of an \linkS4class{eSet} object}
\usage{
esetTsne(
  eset,
  psids = 1:nrow(eset),
  trace = TRUE,
  colorVar = character(),
  color = if (length(colorVar) == 0) "black" else character(),
  shapeVar = character(),
  shape = if (length(shapeVar) == 0) 15 else numeric(),
  sizeVar = character(),
  size = if (length(sizeVar) == 0) {
     ifelse(typePlot[1] == "interactive" &&
    packageInteractivity[1] == "plotly", 20, 2.5)
 } else {
     numeric()
 },
  sizeRange = numeric(),
  alphaVar = character(),
  alpha = if (length(alphaVar) == 0) 1 else numeric(),
  alphaRange = numeric(),
  title = "",
  Rtsne.args = list(perplexity = floor((ncol(eset) - 1)/3), theta = 0.5, dims = 2,
    initial_dims = 50),
  fctTransformDataForInputTsne = NULL,
  symmetryAxes = c("combine", "separate", "none"),
  packageTextLabel = c("ggrepel", "ggplot2"),
  topSamples = 10,
  topSamplesCex = ifelse(typePlot[1] == "interactive" && packageInteractivity[1] ==
    "plotly", 10, 2.5),
  topSamplesVar = character(),
  topSamplesJust = c(0.5, 0.5),
  topSamplesColor = "black",
  includeLegend = TRUE,
  includeLineOrigin = TRUE,
  typePlot = c("static", "interactive"),
  packageInteractivity = c("plotly", "ggvis"),
  figInteractiveSize = c(600, 400),
  ggvisAdjustLegend = TRUE,
  interactiveTooltip = TRUE,
  interactiveTooltipExtraVars = character(),
  returnAnalysis = FALSE,
  returnEsetPlot = FALSE
)
}
\arguments{
\item{eset}{expressionSet (or SummarizedExperiment) object with data}

\item{psids}{featureNames of genes to include in the plot, all by default}

\item{trace}{logical, if TRUE (by default), print some messages during tsne is running}

\item{colorVar}{name of variable (in varLabels of the \code{eset}) used 
for coloring, empty by default}

\item{color}{character or factor with specified color(s) for the points, 
replicated if needed. This is used only if \code{colorVar} is empty.
By default: 'black' if \code{colorVar} is not specified and default 
\code{ggplot} palette otherwise}

\item{shapeVar}{name of variable (in varLabels of the \code{eset}) 
used for the shape, empty by default}

\item{shape}{character or factor with specified shape(s) (pch) for the points, 
replicated if needed. This is used only if \code{shapeVar} is empty.
By default: '15' (filled square) if \code{shapeVar} is not specified 
and default \code{ggplot} shape(s) otherwise}

\item{sizeVar}{name of variable (in varLabels of the \code{eset}) 
used for the size, empty by default}

\item{size}{character or factor with specified size(s) (cex) for the points, 
replicated if needed.
This is used only if \code{sizeVar} is empty.
By default: '2.5' if \code{sizeVar} is not specified (20 for a plotly plot)
and default \code{ggplot} size(s) otherwise}

\item{sizeRange}{size (cex) range used in the plot, possible only 
if the \code{sizeVar} is 'numeric' or 'integer'}

\item{alphaVar}{name of variable (in varLabels of the \code{eset}) 
used for the transparency, empty by default.
This parameter is currently only available for static plot and ggvis (only numeric in this case).}

\item{alpha}{character or factor with specified transparency(s) for the points,
replicated if needed. This is used only if \code{shapeVar} is empty. 
By default: '1' if \code{alphaVar} is not specified and default 
\code{ggplot} alpha otherwise
This parameter is currently only available for static and ggvis.}

\item{alphaRange}{transparency (alpha) range used in the plot, 
possible only if the \code{alphaVar} is 'numeric' or 'integer'
This parameter is currently only available for static and ggvis plot.}

\item{title}{plot title, '' by default}

\item{Rtsne.args}{arguments for the Rtsne function, by default:
perplexite parameter = optimal number of neighbours, 
theta = speed/accuracy trade-off (increase for less accuracy), set to 0.0 for exact TSNE}

\item{fctTransformDataForInputTsne}{function which transform the data in the eSet object before
calling the \code{\link[Rtsne]{Rtsne}} function. 
This should be a function which takes a matrix as input and return a matrix, e.g. the dist function.}

\item{symmetryAxes}{set symmetry for axes, either:
\itemize{
 \item{'combine' (by default): }{both axes are symmetric and with the same limits}
 \item{'separate': }{each axis is symmetric and has its own limits}
 \item{'none': }{axes by default (plot limits)}
}}

\item{packageTextLabel}{package used to label the outlying genes/samples/gene sets,
either \code{ggrepel} (by default, only used if package \code{ggrepel} is available),
or \code{ggplot2}}

\item{topSamples}{numeric indicating which percentile (if <1) or number (if >=1) 
of samples most distant to the origin of the plot to annotate, 
by default: 10 samples are selected
If no samples should be annotated, set this parameter to 0.
Currently available for static plot.}

\item{topSamplesCex}{cex for sample annotation (used when \code{topSamples} > 0)}

\item{topSamplesVar}{variable of the phenoData used to label the samples, 
by default: empty, the sampleNames are used for labelling 
(used when \code{topSample}s > 0)}

\item{topSamplesJust}{text justification for the samples 
(used when \code{topSamples} > 0 and if \code{packageTextLabel} is \code{ggplot2}),
by default: c(0.5, 0.5) so centered}

\item{topSamplesColor}{text color for the samples 
(used when \code{topSamples} > 0), black by default}

\item{includeLegend}{logical if TRUE (by default) include a legend, 
otherwise not}

\item{includeLineOrigin}{if TRUE (by default) include vertical line at 
x = 0 and horizontal line at y = 0}

\item{typePlot}{type of the plot returned, either 'static' (static) or 
interactive' (potentially interactive)}

\item{packageInteractivity}{if \code{typePlot} is 'interactive', 
package used for interactive plot,
either 'plotly' (by default) (by default) or 'ggvis'.}

\item{figInteractiveSize}{vector containing the size of the interactive plot, 
as [width, height]
by default: c(600, 400). This is passed to the \code{width} and 
\code{height} parameters of:
\itemize{
 \item{for plotly plots: }{the \code{\link[plotly]{ggplotly}} function}
 \item{for ggvis plots: }{the \code{ggvis::set_options} function}
}}

\item{ggvisAdjustLegend}{logical, if TRUE (by default) adjust the legends in \code{ggvis} to avoid
overlapping legends when multiple legends}

\item{interactiveTooltip}{logical, if TRUE, add hoover functionality showing
sample annotation (variables used in the plot) in the plot}

\item{interactiveTooltipExtraVars}{name of extra variable(s) 
(in varLabels of the \code{eset}) to add in plotlyEsetPlot to label the samples,
empty by default}

\item{returnAnalysis}{logical, if TRUE (FALSE by default), return also the output of the analysis,
and the outlying samples in the topElements element if any, otherwise only the plot object}

\item{returnEsetPlot}{logical, if TRUE return also the \link{esetPlot} object}
}
\value{
if \code{returnAnalysis} is TRUE, return a list:
\itemize{
 \item{analysis: }{output of the spectral map analysis, whose elements can be given
 to the \code{\link{esetPlotWrapper}} function}
   \itemize{
		\item{dataPlotSamples: }{coordinates of the samples}
		\item{esetUsed: }{expressionSet used in the plot}
	  }
  \item{topElements: }{list with top outlying elements if any, possibly genes, samples and gene sets}
  \item{plot: }{the plot output}
}
otherwise return only the plot
}
\description{
\code{esetTsne} reduces the dimension of the data contained in the \linkS4class{eSet} via t-Distributed Stochastic Neighbor Embedding
 with the \code{\link[Rtsne]{Rtsne}} function and plot the subsequent biplot, possibly with sample annotation contained in the eSet.
}
\examples{
library(ALL)
data(ALL)

## complete example (most of the parameters are optional)

# create custom color palette
colorPalette <- c("dodgerblue", colorRampPalette(c("white","dodgerblue2", "darkblue"))(5)[-1], 
	"red", colorRampPalette(c("white", "red3", "darkred"))(5)[-1])

# create tsne
print(esetTsne(eset = ALL, 
	title = "Acute lymphoblastic leukemia dataset \n Tsne complete",
	colorVar = "BT", color = colorPalette,
	shapeVar = "sex", shape = 15:16,
	sizeVar = "age", sizeRange = c(2, 6),
	symmetryAxes = "separate",
	topSamples = 15, topSamplesVar = "cod", topSamplesColor = "black",
	topSamplesJust = c(1, 0), topSamplesCex = 3)
)
}
\references{
L.J.P. van der Maaten and G.E. Hinton (2008). Visualizing
High-Dimensional Data Using t-SNE. Journal of Machine Learning
Research, 2579--2605
}
\seealso{
the function used internally: \code{\link[Rtsne]{Rtsne}} or \url{http://homepage.tudelft.nl/19j49/t-SNE.html}
 for further explanations about this technique.
}
\author{
Laure Cougnaud
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotTopElements.R
\name{getCoordGeneSets}
\alias{getCoordGeneSets}
\title{extract coordinates gene sets}
\usage{
getCoordGeneSets(dataPlotGenes, geneSets, esetUsed, geneSetsVar = list())
}
\arguments{
\item{dataPlotGenes}{data.frame with two columns 'X' and 'Y' with coordinates for the genes}

\item{geneSets}{geneSets list of gene sets, e.g. gene pathways, output from the 'getGeneSets' function in MLP
the genes IDs must correspond to the sampleNames in the eset}

\item{esetUsed}{expressionSet (or SummarizedExperiment) object with data}

\item{geneSetsVar}{variable of the featureData used to match the genes contained in geneSets,
most probably ENTREZID, if NULL the featureNames of the eSet are used}

\item{...}{Any parameters passed to the \code{\link{getTopElements}}}
}
\value{
data.frame with two columns 'X' and 'Y' with coordinates for the gene sets
}
\description{
extract coordinates gene sets
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetPlot-class.R
\docType{class}
\name{ggvisEsetPlot-class}
\alias{ggvisEsetPlot-class}
\alias{ggvisEsetPlot}
\title{a S4 class for \code{ggvis} plot}
\value{
S4 object of class \code{ggvisEsetPlot}
}
\description{
a S4 class for \code{ggvis} plot
}
\section{Slots}{

\describe{
\item{\code{adjustLegend}}{logical, if TRUE (by default) adjust the legends in \code{ggvis} to avoid
overlapping legends when multiple legends}

\item{\code{alphaRange}}{transparency (alpha) range used in the plot, 
c(0.1, 1) by default.}
}}

\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/esetPlot-class.R
\docType{class}
\name{characterORexpressionOrCall-class}
\alias{characterORexpressionOrCall-class}
\title{S4 Class Union with character/expression/call}
\description{
This is used for the definition of the title/axes labels for the ggplot2 version
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEset.R
\docType{methods}
\name{getDataPlotSamplesWithAnnotation}
\alias{getDataPlotSamplesWithAnnotation}
\alias{getDataPlotSamplesWithAnnotation,esetPlot-method}
\alias{getDataPlotSamplesWithAnnotation,ggvisEsetPlot-method}
\alias{getDataPlotSamplesWithAnnotation,plotlyEsetPlot-method}
\title{get sample data for plot}
\usage{
getDataPlotSamplesWithAnnotation(object)

\S4method{getDataPlotSamplesWithAnnotation}{esetPlot}(object)

\S4method{getDataPlotSamplesWithAnnotation}{ggvisEsetPlot}(object)

\S4method{getDataPlotSamplesWithAnnotation}{plotlyEsetPlot}(object)
}
\arguments{
\item{object}{\link{plotEset} object}
}
\value{
data.frame with 'dataPlotSamples' binded
with variables displayed in the plot
}
\description{
get sample data for plot
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotTopElements.R
\name{getTopElements}
\alias{getTopElements}
\title{create \code{geom_text} object with top genes/sample/pathways}
\usage{
getTopElements(
  top,
  type = c("gene", "sample", "geneSets"),
  var = character(),
  dataPlotGenes = data.frame(),
  dataPlotSamples = data.frame(),
  esetUsed,
  geneSets = list(),
  geneSetsVar = character(),
  geneSetsMaxNChar = numeric()
)
}
\arguments{
\item{top}{numeric, number of top elements}

\item{type}{type of elements to plot, either 'gene', 'sample', or 'geneSets'}

\item{var}{variable used to annotate the elements, only used for 'gene' and 'sample'}

\item{dataPlotGenes}{data.frame with two columns 'X' and 'Y' with coordinates for the genes}

\item{dataPlotSamples}{data.frame with two columns 'X' and 'Y' with coordinates for the samples}

\item{esetUsed}{expressionSet (or SummarizedExperiment) object with data}

\item{geneSets}{list of gene sets, e.g. gene pathways, output from the 'getGeneSets' function in MLP
the genes IDs must correspond to the sampleNames in the eset.
If several gene sets have the same name, they will be combine to extract the top gene sets.}

\item{geneSetsVar}{variable of the featureData used to match the genes contained in geneSets,
most probably ENTREZID, if not specified the featureNames of the eSet are used}

\item{geneSetsMaxNChar}{maximum number of characters for pathway names, by default keep entire names}

\item{returnTopElements}{logical if TRUE (FALSE by default) return the outlying elements}
}
\value{
Data.frame with coordinates and labels of the top elements
}
\description{
create \code{geom_text} object with top genes/sample/pathways
}
\author{
Laure Cougnaud
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotEset-ggvis.R
\name{ggvisPlotEset}
\alias{ggvisPlotEset}
\title{visualize and \link{esetPlot} with the the 'ggvis' package}
\usage{
ggvisPlotEset(object)
}
\arguments{
\item{object}{object of class \link{esetPlot}}
}
\value{
\code{ggvis} plot object
}
\description{
visualize and \link{esetPlot} with the the 'ggvis' package
}
\author{
Laure Cougnaud
}
\keyword{internal}
